from dataclasses import dataclass, field
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class Page:
    """Page
    Args:
        id (int): The id of this lesson page
        lessonid (int): The id of the lesson this page belongs to
        prevpageid (int): The id of the page before this one
        nextpageid (int): The id of the next page in the page sequence
        qtype (int): Identifies the page type of this page
        qoption (int): Used to record page type specific options
        layout (int): Used to record page specific layout selections
        display (int): Used to record page specific display selections
        timecreated (int): Timestamp for when the page was created
        timemodified (int): Timestamp for when the page was last modified
        title (Optional[str]): The title of this page
        contents (Optional[str]): The contents of this page
        contentsformat (Optional[int]): contents format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        displayinmenublock (int): Toggles display in the left menu block
        type (int): The type of the page [question | structure]
        typeid (int): The unique identifier for the page type
        typestring (str): The string that describes this page type
    """
    id: int
    lessonid: int
    prevpageid: int
    nextpageid: int
    qtype: int
    qoption: int
    layout: int
    display: int
    timecreated: int
    timemodified: int
    title: Optional[str]
    contents: Optional[str]
    contentsformat: Optional[int]
    displayinmenublock: int
    type: int
    typeid: int
    typestring: str


@dataclass
class LessonPage:
    """Lesson Page
    Args:
        page (Page): Page fields
        filescount (int): The total number of files attached to the page
        filessizetotal (int): The total size of the files
        answerids (List[int]): List of answers ids (empty for content pages in  Moodle 1.9)
        jumps (List[int]): List of possible page jumps
    """
    page: Page
    filescount: int
    filessizetotal: int
    answerids: List[int] = field(default_factory=list)
    jumps: List[int] = field(default_factory=list)


@dataclass
class Pages(ResponsesFactory[LessonPage]):
    """Pages
    Args:
        pages (List[LessonPage]): LessonPage fields
        warnings (List[Warning]): list of warnings
    """
    pages: List[LessonPage]
    warnings: List[Warning]



from dataclasses import dataclass
from typing import List, Optional


@dataclass
class CustomField:
    """Custom Field
    Args:
        type (str): The type of the custom field - text field, checkbox...
        value (str): The value of the custom field
        name (str): The name of the custom field
        shortname (str): The shortname of the custom field - to be able to build the field class in the code
    """
    type: str
    value: str
    name: str
    shortname: str


@dataclass
class Preference:
    """Preference
    Args:
        name (str): The name of the preferences
        value (str): The value of the preference
    """
    name: str
    value: str


@dataclass
class Group:
    """Group
    Args:
        id (int): group id
        name (str): group name
        description (str): group description
    """
    id: int
    name: str
    description: str


@dataclass
class Role:
    """Role
    Args:
        roleid (int): role id
        name (str): role name
        shortname (str): role shortname
        sortorder (int): role sortorder
    """
    roleid: int
    name: str
    shortname: str
    sortorder: int


@dataclass
class EnrolledCourse:
    """Enrolled Cours
    Args:
        id (int): Id of the course
        fullname (str): Fullname of the course
        shortname (str): Shortname of the course
    """
    id: int
    fullname: str
    shortname: str


@dataclass
class Participant:
    """Participant
    Args:
        id (int): ID of the user
        username (Optional[str]): The username
        firstname (Optional[str]): The first name(s) of the user
        lastname (Optional[str]): The family name of the user
        fullname (str): The fullname of the user
        email (Optional[str]): Email address
        address (Optional[str]): Postal address
        phone1 (Optional[str]): Phone 1
        phone2 (Optional[str]): Phone 2
        icq (Optional[str]): icq number
        skype (Optional[str]): skype id
        yahoo (Optional[str]): yahoo id
        aim (Optional[str]): aim id
        msn (Optional[str]): msn number
        department (Optional[str]): department
        institution (Optional[str]): institution
        idnumber (Optional[str]): The idnumber of the user
        interests (Optional[str]): user interests (separated by commas)
        firstaccess (Optional[int]): first access to the site (0 if never)
        lastaccess (Optional[int]): last access to the site (0 if never)
        suspended (Optional[int]): Suspend user account, either false to enable user login or true to disable it
        description (Optional[str]): User profile description
        descriptionformat (Optional[int]): int format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        city (Optional[str]): Home city of the user
        url (Optional[str]): URL of the user
        country (Optional[str]): Home country code of the user, such as AU or CZ
        profileimageurlsmall (Optional[str]): User image profile URL - small version
        profileimageurl (Optional[str]): User image profile URL - big version
        customfields (List[CustomField]): User custom fields (also known as user profile fields)
        preferences (List[Preference]): Users preferences
        recordid (int): record id
        groups (List[Group]): user groups
        roles (List[Role]): user roles
        enrolledcourses (List[EnrolledCourse]): Courses where the user is enrolled - limited by which courses the user is able to see
        submitted (int): have they submitted their assignment
        requiregrading (int): is their submission waiting for grading
        grantedextension (int): have they been granted an extension
        groupid (Optional[int]): for group assignments this is the group id
        groupname (Optional[str]): for group assignments this is the group name
    """
    id: int
    username: Optional[str]
    firstname: Optional[str]
    lastname: Optional[str]
    fullname: str
    email: Optional[str]
    address: Optional[str]
    phone1: Optional[str]
    phone2: Optional[str]
    icq: Optional[str]
    skype: Optional[str]
    yahoo: Optional[str]
    aim: Optional[str]
    msn: Optional[str]
    department: Optional[str]
    institution: Optional[str]
    idnumber: Optional[str]
    interests: Optional[str]
    firstaccess: Optional[int]
    lastaccess: Optional[int]
    suspended: Optional[int]
    description: Optional[str]
    descriptionformat: Optional[int]
    city: Optional[str]
    url: Optional[str]
    country: Optional[str]
    profileimageurlsmall: Optional[str]
    profileimageurl: Optional[str]
    customfields: List[CustomField]
    preferences: List[Preference]
    recordid: int
    groups: List[Group]
    roles: List[Role]
    enrolledcourses: List[EnrolledCourse]
    submitted: int
    requiregrading: int
    grantedextension: int
    groupid: Optional[int]
    groupname: Optional[str]



from moodle import BaseMoodle


class BaseCompetency(BaseMoodle):
    def add_competency_to_course(self, **kwargs):
        data = self.moodle.get('core_competency_add_competency_to_course',
                               **kwargs)
        return data

    def add_competency_to_plan(self, **kwargs):
        data = self.moodle.get('core_competency_add_competency_to_plan',
                               **kwargs)
        return data

    def add_competency_to_template(self, **kwargs):
        data = self.moodle.get('core_competency_add_competency_to_template',
                               **kwargs)
        return data

    def add_related_competency(self, **kwargs):
        data = self.moodle.get('core_competency_add_related_competency',
                               **kwargs)
        return data

    def approve_plan(self, **kwargs):
        data = self.moodle.get('core_competency_approve_plan', **kwargs)
        return data

    def competency_framework_viewed(self, **kwargs):
        data = self.moodle.get('core_competency_competency_framework_viewed',
                               **kwargs)
        return data

    def competency_viewed(self, **kwargs):
        data = self.moodle.get('core_competency_competency_viewed', **kwargs)
        return data

    def complete_plan(self, **kwargs):
        data = self.moodle.get('core_competency_complete_plan', **kwargs)
        return data

    def count_competencies(self, **kwargs):
        data = self.moodle.get('core_competency_count_competencies', **kwargs)
        return data

    def count_competencies_in_course(self, **kwargs):
        data = self.moodle.get('core_competency_count_competencies_in_course',
                               **kwargs)
        return data

    def count_competencies_in_template(self, **kwargs):
        data = self.moodle.get(
            'core_competency_count_competencies_in_template', **kwargs)
        return data

    def count_competency_frameworks(self, **kwargs):
        data = self.moodle.get('core_competency_count_competency_frameworks',
                               **kwargs)
        return data

    def count_course_module_competencies(self, **kwargs):
        data = self.moodle.get(
            'core_competency_count_course_module_competencies', **kwargs)
        return data

    def count_courses_using_competency(self, **kwargs):
        data = self.moodle.get(
            'core_competency_count_courses_using_competency', **kwargs)
        return data

    def count_templates(self, **kwargs):
        data = self.moodle.get('core_competency_count_templates', **kwargs)
        return data

    def count_templates_using_competency(self, **kwargs):
        data = self.moodle.get(
            'core_competency_count_templates_using_competency', **kwargs)
        return data

    def create_competency(self, **kwargs):
        data = self.moodle.get('core_competency_create_competency', **kwargs)
        return data

    def create_competency_framework(self, **kwargs):
        data = self.moodle.get('core_competency_create_competency_framework',
                               **kwargs)
        return data

    def create_plan(self, **kwargs):
        data = self.moodle.get('core_competency_create_plan', **kwargs)
        return data

    def create_template(self, **kwargs):
        data = self.moodle.get('core_competency_create_template', **kwargs)
        return data

    def create_user_evidence_competency(self, **kwargs):
        data = self.moodle.get(
            'core_competency_create_user_evidence_competency', **kwargs)
        return data

    def delete_competency(self, **kwargs):
        data = self.moodle.get('core_competency_delete_competency', **kwargs)
        return data

    def delete_competency_framework(self, **kwargs):
        data = self.moodle.get('core_competency_delete_competency_framework',
                               **kwargs)
        return data

    def delete_evidence(self, **kwargs):
        data = self.moodle.get('core_competency_delete_evidence', **kwargs)
        return data

    def delete_plan(self, **kwargs):
        data = self.moodle.get('core_competency_delete_plan', **kwargs)
        return data

    def delete_template(self, **kwargs):
        data = self.moodle.get('core_competency_delete_template', **kwargs)
        return data

    def delete_user_evidence(self, **kwargs):
        data = self.moodle.get('core_competency_delete_user_evidence',
                               **kwargs)
        return data

    def delete_user_evidence_competency(self, **kwargs):
        data = self.moodle.get(
            'core_competency_delete_user_evidence_competency', **kwargs)
        return data

    def duplicate_competency_framework(self, **kwargs):
        data = self.moodle.get(
            'core_competency_duplicate_competency_framework', **kwargs)
        return data

    def duplicate_template(self, **kwargs):
        data = self.moodle.get('core_competency_duplicate_template', **kwargs)
        return data

    def get_scale_values(self, **kwargs):
        data = self.moodle.get('core_competency_get_scale_values', **kwargs)
        return data

    def grade_competency(self, **kwargs):
        data = self.moodle.get('core_competency_grade_competency', **kwargs)
        return data

    def grade_competency_in_course(self, **kwargs):
        data = self.moodle.get('core_competency_grade_competency_in_course',
                               **kwargs)
        return data

    def grade_competency_in_plan(self, **kwargs):
        data = self.moodle.get('core_competency_grade_competency_in_plan',
                               **kwargs)
        return data

    def list_competencies(self, **kwargs):
        data = self.moodle.get('core_competency_list_competencies', **kwargs)
        return data

    def list_competencies_in_template(self, **kwargs):
        data = self.moodle.get('core_competency_list_competencies_in_template',
                               **kwargs)
        return data

    def list_competency_frameworks(self, **kwargs):
        data = self.moodle.get('core_competency_list_competency_frameworks',
                               **kwargs)
        return data

    def list_course_competencies(self, **kwargs):
        data = self.moodle.get('core_competency_list_course_competencies',
                               **kwargs)
        return data

    def list_course_module_competencies(self, **kwargs):
        data = self.moodle.get(
            'core_competency_list_course_module_competencies', **kwargs)
        return data

    def list_plan_competencies(self, **kwargs):
        data = self.moodle.get('core_competency_list_plan_competencies',
                               **kwargs)
        return data

    def list_templates(self, **kwargs):
        data = self.moodle.get('core_competency_list_templates', **kwargs)
        return data

    def list_templates_using_competency(self, **kwargs):
        data = self.moodle.get(
            'core_competency_list_templates_using_competency', **kwargs)
        return data

    def list_user_plans(self, **kwargs):
        data = self.moodle.get('core_competency_list_user_plans', **kwargs)
        return data

    def move_down_competency(self, **kwargs):
        data = self.moodle.get('core_competency_move_down_competency',
                               **kwargs)
        return data

    def move_up_competency(self, **kwargs):
        data = self.moodle.get('core_competency_move_up_competency', **kwargs)
        return data

    def plan_cancel_review_request(self, **kwargs):
        data = self.moodle.get('core_competency_plan_cancel_review_request',
                               **kwargs)
        return data

    def plan_request_review(self, **kwargs):
        data = self.moodle.get('core_competency_plan_request_review', **kwargs)
        return data

    def plan_start_review(self, **kwargs):
        data = self.moodle.get('core_competency_plan_start_review', **kwargs)
        return data

    def plan_stop_review(self, **kwargs):
        data = self.moodle.get('core_competency_plan_stop_review', **kwargs)
        return data

    def read_competency(self, **kwargs):
        data = self.moodle.get('core_competency_read_competency', **kwargs)
        return data

    def read_competency_framework(self, **kwargs):
        data = self.moodle.get('core_competency_read_competency_framework',
                               **kwargs)
        return data

    def read_plan(self, **kwargs):
        data = self.moodle.get('core_competency_read_plan', **kwargs)
        return data

    def read_template(self, **kwargs):
        data = self.moodle.get('core_competency_read_template', **kwargs)
        return data

    def read_user_evidence(self, **kwargs):
        data = self.moodle.get('core_competency_read_user_evidence', **kwargs)
        return data

    def remove_competency_from_course(self, **kwargs):
        data = self.moodle.get('core_competency_remove_competency_from_course',
                               **kwargs)
        return data

    def remove_competency_from_plan(self, **kwargs):
        data = self.moodle.get('core_competency_remove_competency_from_plan',
                               **kwargs)
        return data

    def remove_competency_from_template(self, **kwargs):
        data = self.moodle.get(
            'core_competency_remove_competency_from_template', **kwargs)
        return data

    def remove_related_competency(self, **kwargs):
        data = self.moodle.get('core_competency_remove_related_competency',
                               **kwargs)
        return data

    def reopen_plan(self, **kwargs):
        data = self.moodle.get('core_competency_reopen_plan', **kwargs)
        return data

    def reorder_course_competency(self, **kwargs):
        data = self.moodle.get('core_competency_reorder_course_competency',
                               **kwargs)
        return data

    def reorder_plan_competency(self, **kwargs):
        data = self.moodle.get('core_competency_reorder_plan_competency',
                               **kwargs)
        return data

    def reorder_template_competency(self, **kwargs):
        data = self.moodle.get('core_competency_reorder_template_competency',
                               **kwargs)
        return data

    def request_review_of_user_evidence_linked_competencies(self, **kwargs):
        data = self.moodle.get(
            'core_competency_request_review_of_user_evidence_linked_competencies',  # NOQA
            **kwargs)
        return data

    def search_competencies(self, **kwargs):
        data = self.moodle.get('core_competency_search_competencies', **kwargs)
        return data

    def set_course_competency_ruleoutcome(self, **kwargs):
        data = self.moodle.get(
            'core_competency_set_course_competency_ruleoutcome', **kwargs)
        return data

    def set_parent_competency(self, **kwargs):
        data = self.moodle.get('core_competency_set_parent_competency',
                               **kwargs)
        return data

    def template_has_related_data(self, **kwargs):
        data = self.moodle.get('core_competency_template_has_related_data',
                               **kwargs)
        return data

    def template_viewed(self, **kwargs):
        data = self.moodle.get('core_competency_template_viewed', **kwargs)
        return data

    def unapprove_plan(self, **kwargs):
        data = self.moodle.get('core_competency_unapprove_plan', **kwargs)
        return data

    def unlink_plan_from_template(self, **kwargs):
        data = self.moodle.get('core_competency_unlink_plan_from_template',
                               **kwargs)
        return data

    def update_competency(self, **kwargs):
        data = self.moodle.get('core_competency_update_competency', **kwargs)
        return data

    def update_competency_framework(self, **kwargs):
        data = self.moodle.get('core_competency_update_competency_framework',
                               **kwargs)
        return data

    def update_course_competency_settings(self, **kwargs):
        data = self.moodle.get(
            'core_competency_update_course_competency_settings', **kwargs)
        return data

    def update_plan(self, **kwargs):
        data = self.moodle.get('core_competency_update_plan', **kwargs)
        return data

    def update_template(self, **kwargs):
        data = self.moodle.get('core_competency_update_template', **kwargs)
        return data

    def user_competency_cancel_review_request(self, **kwargs):
        data = self.moodle.get(
            'core_competency_user_competency_cancel_review_request', **kwargs)
        return data

    def user_competency_plan_viewed(self, **kwargs):
        data = self.moodle.get('core_competency_user_competency_plan_viewed',
                               **kwargs)
        return data

    def user_competency_request_review(self, **kwargs):
        data = self.moodle.get(
            'core_competency_user_competency_request_review', **kwargs)
        return data

    def user_competency_start_review(self, **kwargs):
        data = self.moodle.get('core_competency_user_competency_start_review',
                               **kwargs)
        return data

    def user_competency_stop_review(self, **kwargs):
        data = self.moodle.get('core_competency_user_competency_stop_review',
                               **kwargs)
        return data

    def user_competency_viewed(self, **kwargs):
        data = self.moodle.get('core_competency_user_competency_viewed',
                               **kwargs)
        return data

    def user_competency_viewed_in_course(self, **kwargs):
        data = self.moodle.get(
            'core_competency_user_competency_viewed_in_course', **kwargs)
        return data

    def user_competency_viewed_in_plan(self, **kwargs):
        data = self.moodle.get(
            'core_competency_user_competency_viewed_in_plan', **kwargs)
        return data



from .activity_completion_status import (
    ActivityCompletion,
    ActivityCompletionStatus,
)
from .course_completion_status import (
    CourseCompletionDetail,
    CourseCompletion,
    CompletionStatus,
    CourseCompletionStatus,
)
from .status_completed import StatusCompletion

from .base import BaseCompletion

__all__ = [
    'ActivityCompletion', 'ActivityCompletionStatus', 'CourseCompletionDetail',
    'CourseCompletion', 'CompletionStatus', 'CourseCompletionStatus',
    'StatusCompletion', 'BaseCompletion'
]



from dataclasses import dataclass
from typing import List, Optional, Union
from moodle import MoodleObject, ResponsesFactory, Warning


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Discussion(MoodleObject):
    """Discussion
    Args:
        id (int): Post id
        name (str): Discussion name
        groupid (int): Group id
        timemodified (int): Time modified
        usermodified (int): The id of the user who last modified
        timestart (int): Time discussion can start
        timeend (int): Time discussion ends
        discussion (int): Discussion id
        parent (int): Parent id
        userid (int): User who started the discussion id
        created (int): Creation time
        modified (int): Time modified
        mailed (int): Mailed?
        subject (str): The post subject
        message (str): The post message
        messageformat (int): message format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        messagetrust (int): Can we trust?
        messageinlinefiles (List[File]): post message inline files
        attachment (str): Has attachments?
        attachments (List[File]): attachments
        totalscore (int): The post message total score
        mailnow (int): Mail now?
        userfullname (str): Post author full name
        usermodifiedfullname (str): Post modifier full name
        userpictureurl (str): Post author picture.
        usermodifiedpictureurl (str): Post modifier picture.
        numreplies (int): The number of replies in the discussion
        numunread (int): The number of unread discussions.
        pinned (int): Is the discussion pinned
        locked (int): Is the discussion locked
        starred (int): Is the discussion starred
        canreply (int): Can the user reply to the discussion
        canlock (int): Can the user lock the discussion
        canfavourite (int): Can the user star the discussion
    """
    id: int
    name: str
    groupid: int
    timemodified: int
    usermodified: int
    timestart: int
    timeend: int
    discussion: int
    parent: int
    userid: int
    created: int
    modified: int
    mailed: int
    subject: str
    message: str
    messageformat: int
    messagetrust: int
    messageinlinefiles: List[File]
    attachment: str
    attachments: List[File]
    totalscore: int
    mailnow: int
    userfullname: str
    usermodifiedfullname: str
    userpictureurl: str
    usermodifiedpictureurl: str
    numreplies: int
    numunread: int
    pinned: int
    locked: int
    starred: int
    canreply: int
    canlock: int
    canfavourite: int


@dataclass
class Discussions(ResponsesFactory[Discussion]):
    """Discussions
    Args:
        discussions (List[Discussion]): list of Discussion
        warnings (List[Warning]): list of Warning

    Returns:
        Discussions: List of Discussion
    """
    discussions: List[Discussion]
    warnings: List[Warning]

    @property
    def items(self) -> List[Discussion]:
        return self.discussions

    @dataclass
    class Option:
        """Discussion Option
        Args:
            name (str): The allowed keys (value format) are:
                            discussionsubscribe (bool); subscribe to the discussion?, default to true
                            discussionpinned    (bool); is the discussion pinned, default to false
                            inlineattachmentsid (int); the draft file area id for inline attachments
                            attachmentsid       (int); the draft file area id for attachments
            value (str): The value of the option, This param is validated in the external function.
        """
        name: str
        value: Union[bool, int]

    @dataclass
    class New:
        """Response of add_discussion
        Args:
            discussionid (int): New Discussion ID
            warnings (List[Warning]): list of warnings
        """
        discussionid: int
        warnings: List[Warning]

    @dataclass
    class CanAdd:
        """Response of can_add_discussion
        Args:
            status (int): True if the user can add discussions, false otherwise.
            canpindiscussions (Optional[int]): True if the user can pin discussions, false otherwise.
            cancreateattachment (Optional[int]): True if the user can add attachments, false otherwise.
            warnings (List[Warning]): list of warnings
        """
        status: int
        canpindiscussions: Optional[int]
        cancreateattachment: Optional[int]
        warnings: List[Warning]



from datetime import datetime
from typing import List, Optional, Union
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import Pages, Lessons, OneLesson


class BaseLesson(BaseMoodle):
    def finish_attempt(self,
                       lessonid: int,
                       password: str = '',
                       outoftime: Optional[Union[datetime, int]] = None,
                       review: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_finish_attempt',
            lessonid=lessonid,
            password=password,
            outoftime=outoftime or '',
            review=review or '',
        )
        return res

    def get_attempts_overview(self, lessonid: int, groupid: int = 0):
        res = self.moodle.post(
            'mod_lesson_get_attempts_overview',
            lessonid=lessonid,
            groupid=groupid,
        )
        return res

    def get_content_pages_viewed(self,
                                 lessonid: int,
                                 lessonattempt: int,
                                 userid: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_get_content_pages_viewed',
            lessonid=lessonid,
            lessonattempt=lessonattempt,
            userid=userid,
        )
        return res

    def get_lesson(self, lessonid: int, password: str = '') -> OneLesson:
        """Return information of a given lesson.

        Args:
            lessonid (int): lesson instance id
            password (str, optional): lesson password. Defaults to ''.

        Returns:
            OneLesson: Object containing Lesson
        """
        res = self.moodle.post(
            'mod_lesson_get_lesson',
            lessonid=lessonid,
            password=password,
        )
        return from_dict(OneLesson, res)

    def get_lesson_access_information(self, lessonid: int):
        res = self.moodle.post(
            'mod_lesson_get_lesson_access_information',
            lessonid=lessonid,
        )
        return res

    def get_lessons_by_courses(self,
                               courseids: Optional[List[int]] = None
                               ) -> Lessons:
        """Returns a list of lessons in a provided list of courses, if no list is provided all lessons that the user can view will be returned.

        Args:
            courseids (Optional[List[int]], optional): Array of course ids. Defaults to [].

        Returns:
            Lessons: List of Lesson
        """
        res = self.moodle.post(
            'mod_lesson_get_lessons_by_courses',
            courseids=courseids or [],
        )
        return from_dict(Lessons, res)

    def get_page_data(self,
                      lessonid: int,
                      pageid: int,
                      password: str = '',
                      review: str = '',
                      returncontents: str = ''):
        res = self.moodle.post(
            'mod_lesson_get_page_data',
            lessonid=lessonid,
            pageid=pageid,
            password=password,
            review=review,
            returncontents=returncontents,
        )
        return res

    def get_pages(self, lessonid: int, password: str = '') -> Pages:
        """Return the list of pages in a lesson (based on the user permissions).

        Args:
            lessonid (int): lesson instance id
            password (str, optional): optional password (the lesson may be protected). Defaults to ''.

        Returns:
            Pages: List of LessonPage
        """
        res = self.moodle.post(
            'mod_lesson_get_pages',
            lessonid=lessonid,
            password=password,
        )
        return from_dict(Pages, res)

    def get_pages_possible_jumps(self, lessonid: int):
        res = self.moodle.post(
            'mod_lesson_get_pages_possible_jumps',
            lessonid=lessonid,
        )
        return res

    def get_questions_attempts(self,
                               lessonid: int,
                               attempt: int,
                               correct: Optional[int] = None,
                               pageid: Optional[int] = None,
                               userid: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_get_questions_attempts',
            lessonid=lessonid,
            attempt=attempt,
            correct=correct or '',
            pageid=pageid,
            userid=userid,
        )
        return res

    def get_user_attempt(self, lessonid: int, userid: int, lessonattempt: int):
        res = self.moodle.post(
            'mod_lesson_get_user_attempt',
            lessonid=lessonid,
            userid=userid,
            lessonattempt=lessonattempt,
        )
        return res

    def get_user_attempt_grade(self,
                               lessonid: int,
                               lessonattempt: int,
                               userid: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_get_user_attempt_grade',
            lessonid=lessonid,
            lessonattempt=lessonattempt,
            userid=userid,
        )
        return res

    def get_user_grade(self, lessonid: int, userid: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_get_user_grade',
            lessonid=lessonid,
            userid=userid,
        )
        return res

    def get_user_timers(self, lessonid: int, userid: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_get_user_timers',
            lessonid=lessonid,
            userid=userid,
        )
        return res

    def launch_attempt(self,
                       lessonid: int,
                       password: str = '',
                       pageid: int = 0,
                       review: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_launch_attempt',
            lessonid=lessonid,
            password=password,
            pageid=pageid,
            review=review,
        )
        return res

    def process_page(self,
                     lessonid: int,
                     pageid: int,
                     data: List[dict],
                     password: str = '',
                     review: Optional[int] = None):
        res = self.moodle.post(
            'mod_lesson_process_page',
            lessonid=lessonid,
            pageid=pageid,
            data=data,
            password=password,
            review=review or '',
        )
        return res

    def view_lesson(self, lessonid: int, password: str = ''):
        res = self.moodle.post(
            'mod_lesson_view_lesson',
            lessonid=lessonid,
            password=password,
        )
        return res



from .badges import BaseBadges
from .block import BaseBlock
from .blog import BaseBlog
from .calendar import BaseCalendar
from .cohort import BaseCohort
from .comment import BaseComment
from .competency import BaseCompetency
from .completion import BaseCompletion
from .course import BaseCourse
from .message import BaseMessage
from .notes import BaseNotes
from .user import BaseUser
from .webservice import BaseWebservice

from .core import Core

__all__ = [
    'BaseBadges',
    'BaseBlock',
    'BaseBlog',
    'BaseCalendar',
    'BaseCohort',
    'BaseComment',
    'BaseCompetency',
    'BaseCompletion',
    'BaseCourse',
    'BaseMessage',
    'BaseNotes',
    'BaseUser',
    'BaseWebservice',
    'Core',
]



from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import ActivityCompletionStatus, CourseCompletionStatus, StatusCompletion


class BaseCompletion(BaseMoodle):
    def get_activities_completion_status(
            self, courseid: int, userid: int) -> ActivityCompletionStatus:
        """Return the activities completion status for a user in a course.

        Args:
            courseid (int): Course ID
            userid (int): User ID

        Returns:
            CompletionStatus: Activities completion status for a user in a course.
        """
        res = self.moodle.post(
            'core_completion_get_activities_completion_status',
            courseid=courseid,
            userid=userid,
        )
        return from_dict(ActivityCompletionStatus, res)

    def get_course_completion_status(self, courseid: int,
                                     userid: int) -> CourseCompletionStatus:
        """Returns course completion status.

        Args:
            courseid (int): Course ID
            userid (int): User ID

        Returns:
            CourseCompletionStatus: Course completion status.
        """
        res = self.moodle.post(
            'core_completion_get_course_completion_status',
            courseid=courseid,
            userid=userid,
        )
        return from_dict(CourseCompletionStatus, res)

    def mark_course_self_completed(self, courseid: int) -> StatusCompletion:
        """Update the course completion status for the current user (if course self-completion is enabled).

        Args:
            courseid (int): Course ID

        Returns:
            SelfCompleted: Response
        """
        res = self.moodle.post('core_completion_mark_course_self_completed',
                               courseid=courseid)
        return from_dict(StatusCompletion, res)

    def update_activity_completion_status_manually(
            self, cmid: int, completed: int) -> StatusCompletion:
        """Update completion status for the current user in an activity, only for activities with manual tracking.

        Args:
            cmid (int): course module id
            completed (int): activity completed or not

        Returns:
            StatusCompletion: Response
        """
        res = self.moodle.post(
            'core_completion_update_activity_completion_status_manually',
            cmid=cmid,
            completed=completed,
        )
        return from_dict(StatusCompletion, res)



from typing import List, Optional
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import Content, FunctionsResponses, Key, MobileConfig, MobilePlugin, MobilePublicConfig


class BaseMobile(BaseMoodle):
    def call_external_functions(
            self,
            requests: List[FunctionsResponses.Request]) -> FunctionsResponses:
        """Call multiple external functions and return all responses.

        Args:
            requests (List[FunctionsResponses.Request]): Functions requests

        Returns:
            FunctionsResponses: Functions Response
        """
        res = self.moodle.post('tool_mobile_call_external_functions')
        return from_dict(FunctionsResponses, res)

    def get_autologin_key(self, privatetoken: str) -> Key:
        """Creates an auto-login key for the current user.
        Is created only in https sites and is restricted by time, ip address
        and only works if the request comes from the Moodle mobile or desktop app.

        Args:
            privatetoken (str): Private token, usually generated by login/token.php

        Returns:
            Key: Key object containing auto-login
        """
        res = self.moodle.post('tool_mobile_get_autologin_key',
                               privatetoken=privatetoken)
        return from_dict(Key, res)

    def get_config(self, section: Optional[str] = None) -> MobileConfig:
        """Returns a list of the site configurations, filtering by section.

        Args:
            section (str, optional): Settings section name. Defaults to None.

        Returns:
            MobileConfig: Returns a list of the site configurations, filtering by section.
        """
        res = self.moodle.get('tool_mobile_get_config', section=section or '')
        return from_dict(MobileConfig, res)

    def get_content(self,
                    component: str,
                    method: str,
                    args: Optional[Content.Args] = None) -> Content:
        """Get a piece of content to be displayed in the Mobile app.

        Args:
            component (str): Component where the class is e.g. mod_assign.
            method (str): Method to execute in class \\$component\\output\\mobile.
            args (Optional[Content.Args], optional): Args for the method are optional.. Defaults to None.

        Returns:
            Content: Returns a piece of content to be displayed in the Mobile app.
        """
        res = self.moodle.post(
            'tool_mobile_get_content',
            component=component,
            method=method,
            args=args,
        )
        return from_dict(Content, res)

    def get_plugins_supporting_mobile(self) -> List[MobilePlugin]:
        """Get a list of Moodle plugins supporting the mobile app.

        Returns:
            List[MobilePlugin]: Returns a list of Moodle plugins supporting the mobile app.
        """
        res = self.moodle.get('tool_mobile_get_plugins_supporting_mobile')
        return [from_dict(MobilePlugin, data)
                for data in res['plugins']] if 'plugins' in res else []

    def get_public_config(self) -> MobilePublicConfig:
        """Get a list of the site public settings, those not requiring authentication.

        Returns:
            MobilePublicConfig: Returns a list of the site public settings, those not requiring authentication.
        """
        data = self.moodle.get('tool_mobile_get_public_config')
        return from_dict(MobilePublicConfig, data)



from dataclasses import dataclass
from typing import Optional


@dataclass
class Warning:
    item: Optional[str]
    itemid: Optional[int]
    warningcode: str
    message: str

    def __str__(self) -> str:
        return self.message



from datetime import datetime
from typing import List, Optional, Union
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import (AccessInformation, AllowedEventTypes, Event, Events,
               CourseEvents, ActionEventCourses, DayView, MonthlyView)


class BaseCalendar(BaseMoodle):
    def create_calendar_events(self, events: List[Events.Create]) -> Events:
        res = self.moodle.get('core_calendar_create_calendar_events',
                              events=events)
        return from_dict(Events, res)

    def delete_calendar_events(self, events: List[Events.Delete]) -> None:
        self.moodle.get('core_calendar_delete_calendar_events')
        return None

    def get_action_events_by_course(self,
                                    courseid: int,
                                    timesortfrom: Optional[Union[datetime,
                                                                 int]] = None,
                                    timesortto: Optional[Union[datetime,
                                                               int]] = None,
                                    aftereventid: int = 0,
                                    limitnum: int = 20) -> CourseEvents:
        res = self.moodle.get('core_calendar_get_action_events_by_course',
                              courseid=courseid,
                              timesortfrom=timesortfrom,
                              timesortto=timesortto,
                              aftereventid=aftereventid,
                              limitnum=limitnum)
        return from_dict(CourseEvents, res)

    def get_action_events_by_courses(self,
                                     courseids: List[int],
                                     timesortfrom: Optional[Union[datetime,
                                                                  int]] = None,
                                     timesortto: Optional[Union[datetime,
                                                                int]] = None,
                                     limitnum: int = 10) -> ActionEventCourses:
        res = self.moodle.get('core_calendar_get_action_events_by_courses',
                              courseids=courseids,
                              timesortfrom=timesortfrom,
                              limitnum=limitnum)
        return from_dict(ActionEventCourses, res)

    def get_action_events_by_timesort(
            self,
            timesortfrom: Optional[Union[datetime, int]] = 0,
            timesortto: Optional[Union[datetime, int]] = None,
            aftereventid: int = 0,
            limitnum: int = 20,
            limittononsuspendedevents: Optional[int] = None,
            userid: Optional[int] = None) -> CourseEvents:
        res = self.moodle.get(
            'core_calendar_get_action_events_by_timesort',
            timesortfrom=timesortfrom,
            timesortto=timesortto,
            aftereventid=aftereventid,
            limitnum=limitnum,
            limittononsuspendedevents=limittononsuspendedevents,
            userid=userid,
        )
        return from_dict(CourseEvents, res)

    def get_allowed_event_types(self, courseid: int = 0) -> AllowedEventTypes:
        res = self.moodle.get('core_calendar_get_allowed_event_types',
                              courseid=courseid)
        return from_dict(AllowedEventTypes, res)

    def get_calendar_access_information(self,
                                        courseid: int = 0
                                        ) -> AccessInformation:
        res = self.moodle.get('core_calendar_get_calendar_access_information',
                              courseid=courseid)
        return from_dict(AccessInformation, res)

    def get_calendar_day_view(self,
                              year: int,
                              month: int,
                              day: int,
                              courseid: int = 1,
                              categoryid: Optional[int] = None) -> DayView:
        res = self.moodle.get(
            'core_calendar_get_calendar_day_view',
            year=year,
            month=month,
            day=day,
            courseid=courseid,
            categoryid=categoryid,
        )
        return from_dict(DayView, res)

    def get_calendar_event_by_id(self, eventid: int) -> CourseEvents:
        res = self.moodle.get('core_calendar_get_calendar_event_by_id',
                              eventid=eventid)
        return from_dict(CourseEvents, res)

    def get_calendar_events(
            self,
            events: Optional[Events.Details] = None,
            options: Optional[Events.Options] = None) -> Events:
        res = self.moodle.get('core_calendar_get_calendar_events',
                              events=events if events else {},
                              options=options if options else {})
        return from_dict(Events, res)

    def get_calendar_monthly_view(self,
                                  year: int,
                                  month: int,
                                  courseid: int = 1,
                                  categoryid: Optional[int] = None,
                                  includenavigation: int = 1,
                                  mini: Optional[int] = None) -> MonthlyView:
        res = self.moodle.get('core_calendar_get_calendar_monthly_view')
        return from_dict(MonthlyView, res)

    def get_calendar_upcoming_view(
            self,
            courseid: int,
            categoryid: Optional[int] = None) -> CourseEvents:
        res = self.moodle.get('core_calendar_get_calendar_upcoming_view')
        return from_dict(CourseEvents, res)

    def get_timestamps(self):
        res = self.moodle.get('core_calendar_get_timestamps')
        return res

    def submit_create_update_form(self,
                                  formdata: str) -> CourseEvents.EventForm:
        """Submit form data for event form

        Args:
            formdata (str): The data from the event form

        Returns:
            CourseEvents.EventForm: Event form
        """
        res = self.moodle.get('core_calendar_submit_create_update_form')
        return from_dict(CourseEvents.EventForm, res)

    def update_event_start_day(self, eventid: int,
                               daytimestamp: Union[datetime, int]) -> Event:
        """Update the start day (but not time) for an event.

        Args:
            eventid (int): Id of event to be updated
            daytimestamp (Union[datetime, int]): Timestamp for the new start day

        Returns:
            Event: Updated event
        """
        res = self.moodle.get('core_calendar_update_event_start_day',
                              eventid=eventid,
                              daytimestamp=daytimestamp)
        return from_dict(Event, res['event'])



from dataclasses import dataclass
from typing import List, Optional
from . import DateView, Period, CourseEvent


@dataclass
class Day:
    """Day
    seconds (int): seconds
    minutes (int): minutes
    hours (int): hours
    mday (int): mday
    wday (int): wday
    year (int): year
    yday (int): yday
    istoday (int): istoday
    isweekend (int): isweekend
    timestamp (int): timestamp
    neweventtimestamp (int): neweventtimestamp
    viewdaylink (Optional[str]): viewdaylink
    viewdaylinktitle (Optional[str]): viewdaylinktitle
    events (List[CourseEvent]): events
    hasevents (int): hasevents
    calendareventtypes (List[str]): calendareventtypes
    previousperiod (int): previousperiod
    nextperiod (int): nextperiod
    navigation (str): navigation
    haslastdayofevent (int): haslastdayofevent
    popovertitle (str): popovertitle
    """
    seconds: int
    minutes: int
    hours: int
    mday: int
    wday: int
    year: int
    yday: int
    istoday: int
    isweekend: int
    timestamp: int
    neweventtimestamp: int
    viewdaylink: Optional[str]
    viewdaylinktitle: Optional[str]
    events: List[CourseEvent]
    hasevents: int
    calendareventtypes: List[str]
    previousperiod: int
    nextperiod: int
    navigation: str
    haslastdayofevent: int
    popovertitle: str


@dataclass
class Week:
    """Week
    prepadding (List[int]): prepadding
    postpadding (List[int]): postpadding
    days (List[Day]): days
    """
    prepadding: List[int]
    postpadding: List[int]
    days: List[Day]


@dataclass
class DayName:
    """Day Name
    dayno (int): dayno
    shortname (str): shortname
    fullname (str): fullnames
    """
    dayno: int
    shortname: str
    fullname: str


@dataclass
class MonthlyView:
    """Monthly View
    url (str): url
    courseid (int): courseid
    categoryid (Optional[int]): categoryid
    filter_selector (Optional[str]): filter_selector
    weeks (Week): weeks
    daynames (DayName): daynames
    date (DateView): date
    periodname (str): periodname
    includenavigation (int): includenavigation
    initialeventsloaded (int): initialeventsloaded
    previousperiod (Period): previousperiod
    previousperiodlink (str): previousperiodlink
    previousperiodname (str): previousperiodname
    nextperiod (Period): nextperiod
    nextperiodname (str): nextperiodname
    nextperiodlink (str): nextperiodlink
    larrow (str): larrow
    rarrow (str): rarrow
    defaulteventcontext (int): defaulteventcontext
    """
    url: str
    courseid: int
    categoryid: Optional[int]
    filter_selector: Optional[str]
    weeks: Week
    daynames: DayName
    date: DateView
    periodname: str
    includenavigation: int
    initialeventsloaded: int
    previousperiod: Period
    previousperiodlink: str
    previousperiodname: str
    nextperiod: Period
    nextperiodname: str
    nextperiodlink: str
    larrow: str
    rarrow: str
    defaulteventcontext: int



from __future__ import annotations
from typing import Optional, Type, TypeVar, TYPE_CHECKING
from moodle.utils.helper import from_dict
if TYPE_CHECKING:
    from moodle import Moodle

T = TypeVar('T')


class MoodleObject(object):
    def __post_init__(self):
        self._moodle: Optional[Moodle] = None

    @property
    def moodle(self) -> Optional[Moodle]:
        return self._moodle

    @classmethod
    def from_data(cls: Type[T], data: dict, moodle: Moodle) -> T:
        self = from_dict(cls, data)
        setattr(self, '_moodle', moodle)
        return self

    def __str__(self) -> str:
        return getattr(self, 'name', repr(self))



from .page import (
    Page,
    LessonPage,
    Pages,
)
from .lesson import (
    Lesson,
    OneLesson,
    Lessons,
)

from .base import BaseLesson

__all__ = [
    'Page', 'LessonPage', 'Pages', 'Lesson', 'OneLesson', 'Lessons',
    'BaseLesson'
]



from dataclasses import dataclass
from typing import List
from moodle import Warning, ResponsesFactory


@dataclass
class AllowedEventTypes(ResponsesFactory[str]):
    """The type of events a user can create in the given course.
    Constructor arguments:
    allowedeventtypes: List[str]
    warnings: List[Warning]
    """
    allowedeventtypes: List[str]
    warnings: List[Warning]



from dataclasses import dataclass, field
from typing import List
from moodle import Warning


@dataclass
class ViewEntry:
    """the blog_entries_viewed event.
    Constructor arguments:
    params: status (bool): status: true if success
    params: warnings (List[Warning]): list of warnings
    """
    status: bool
    warnings: List[Warning] = field(default_factory=list)



from moodle import BaseMoodle
from . import BaseAssign, BaseForum, BaseLesson, BaseResource


class Mod(BaseMoodle):
    def __post_init__(self, moodle) -> None:
        self._assign = BaseAssign(moodle)
        self._forum = BaseForum(moodle)
        self._lesson = BaseLesson(moodle)
        self._resource = BaseResource(moodle)

    @property
    def assign(self) -> BaseAssign:
        return self._assign

    @property
    def forum(self) -> BaseForum:
        return self._forum

    @property
    def lesson(self) -> BaseLesson:
        return self._lesson

    @property
    def resource(self) -> BaseResource:
        return self._resource



from .assign import BaseAssign
from .forum import BaseForum
from .lesson import BaseLesson
from .resource import BaseResource

from .mod import Mod

__all__ = ['BaseAssign', 'BaseForum', 'BaseLesson', 'BaseResource', 'Mod']



from dataclasses import dataclass
from typing import List, Optional
from moodle import ResponsesFactory, Warning
from . import AdvanceGradingData, PluginData


@dataclass
class Grade:
    """Grade
    Args:
        id (int): grade id
        assignment (Optional[int]): assignment id
        userid (int): student id
        attemptnumber (int): attempt number
        timecreated (int): grade creation time
        timemodified (int): grade last modified time
        grader (int): grader, -1 if grader is hidden
        grade (str): grade
        gradefordisplay (Optional[str]): grade rendered into a format suitable for display
    """
    id: int
    assignment: Optional[int]
    userid: int
    attemptnumber: int
    timecreated: int
    timemodified: int
    grader: int
    grade: str
    gradefordisplay: Optional[str]


@dataclass
class GradeAssignment(ResponsesFactory[Grade]):
    """GradeAssignment
    Args:
        assignmentid (int): assignment id
        grades (List[Grade]): grade information
    """
    assignmentid: int
    grades: List[Grade]

    @property
    def items(self) -> List[Grade]:
        return self.grades


@dataclass
class Grades(ResponsesFactory[GradeAssignment]):
    """Grades
    Args:
        assignments (List[GradeAssignment]): list of assignment grade information
        warnings (List[Warning]): list of warnings
    """
    assignments: List[GradeAssignment]
    warnings: List[Warning]

    @property
    def items(self) -> List[GradeAssignment]:
        return self.assignments

    @dataclass
    class Create:
        """Grades for create grades
        Args:
            userid (int): The student id to operate on
            grade (float): The new grade for this user. Ignored if advanced grading used
            attemptnumber (int): The attempt number (-1 means latest attempt)
            addattempt (int): Allow another attempt if manual attempt reopen method
            workflowstate (str): The next marking workflow state
            plugindata (Optional[PluginData]): default {}
            advancedgradingdata (Optional[AdvanceGradingData]): default {}
        """
        userid: int
        grade: float
        attemptnumber: int
        addattempt: int
        workflowstate: str
        plugindata: Optional[PluginData]
        advancedgradingdata: Optional[AdvanceGradingData]



from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for the external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Lesson:
    """Lesson
    Args:
        id (int): Standard Moodle primary key.
        course (int): Foreign key reference to the course this lesson is part of.
        coursemodule (int): Course module id.
        name (str): Lesson name.
        intro (Optional[str]): Lesson introduction text.
        introformat (int): Default untuk "1" intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        practice (Optional[int]): Practice lesson?
        modattempts (Optional[int]): Allow student review?
        usepassword (Optional[int]): Password protected lesson?
        password (Optional[str]): Password
        dependency (Optional[int]): Dependent on (another lesson id)
        conditions (Optional[str]): Conditions to enable the lesson
        grade (Optional[int]): The total that the grade is scaled to be out of
        custom (Optional[int]): Custom scoring?
        ongoing (Optional[int]): Display ongoing score?
        usemaxgrade (Optional[int]): How to calculate the final grade
        maxanswers (Optional[int]): Maximum answers per page
        maxattempts (Optional[int]): Maximum attempts
        review (Optional[int]): Provide option to try a question again
        nextpagedefault (Optional[int]): Action for a correct answer
        feedback (Optional[int]): Display default feedback
        minquestions (Optional[int]): Minimum number of questions
        maxpages (Optional[int]): Number of pages to show
        timelimit (Optional[int]): Time limit
        retake (Optional[int]): Re-takes allowed
        activitylink (Optional[int]): Id of the next activity to be linked once the lesson is completed
        mediafile (Optional[str]): Local file path or full external URL
        mediaheight (Optional[int]): Popup for media file height
        mediawidth (Optional[int]): Popup for media with
        mediaclose (Optional[int]): Display a close button in the popup?
        slideshow (Optional[int]): Display lesson as slideshow
        width (Optional[int]): Slideshow width
        height (Optional[int]): Slideshow height
        bgcolor (Optional[str]): Slideshow bgcolor
        displayleft (Optional[int]): Display left pages menu?
        displayleftif (Optional[int]): Minimum grade to display menu
        progressbar (Optional[int]): Display progress bar?
        available (Optional[int]): Available from
        deadline (Optional[int]): Available until
        timemodified (Optional[int]): Last time settings were updated
        completionendreached (Optional[int]): Require end reached for completion?
        completiontimespent (Optional[int]): Student must do this activity at least for
        allowofflineattempts (int): Whether to allow the lesson to be attempted offline in the mobile app
        introfiles (List[File]): introfiles
        mediafiles (List[File]): mediafiles
    """
    id: int
    course: int
    coursemodule: int
    name: str
    intro: Optional[str]
    introformat: int
    practice: Optional[int]
    modattempts: Optional[int]
    usepassword: Optional[int]
    password: Optional[str]
    dependency: Optional[int]
    conditions: Optional[str]
    grade: Optional[int]
    custom: Optional[int]
    ongoing: Optional[int]
    usemaxgrade: Optional[int]
    maxanswers: Optional[int]
    maxattempts: Optional[int]
    review: Optional[int]
    nextpagedefault: Optional[int]
    feedback: Optional[int]
    minquestions: Optional[int]
    maxpages: Optional[int]
    timelimit: Optional[int]
    retake: Optional[int]
    activitylink: Optional[int]
    mediafile: Optional[str]
    mediaheight: Optional[int]
    mediawidth: Optional[int]
    mediaclose: Optional[int]
    slideshow: Optional[int]
    width: Optional[int]
    height: Optional[int]
    bgcolor: Optional[str]
    displayleft: Optional[int]
    displayleftif: Optional[int]
    progressbar: Optional[int]
    available: Optional[int]
    deadline: Optional[int]
    timemodified: Optional[int]
    completionendreached: Optional[int]
    completiontimespent: Optional[int]
    allowofflineattempts: int
    introfiles: List[File]
    mediafiles: List[File]


@dataclass
class OneLesson:
    lesson: Lesson
    warnings: List[Warning]


@dataclass
class Lessons(ResponsesFactory[Lesson]):
    lessons: List[Lesson]
    warnings: List[Warning]

    @property
    def items(self) -> List[Lesson]:
        return self.lessons



from dataclasses import dataclass
from .notification_info import NotificationInfo


@dataclass
class NotificationProcessor:
    displayname: str
    name: str
    locked: bool
    userconfigured: int
    loggedin: NotificationInfo
    loggedoff: NotificationInfo



from typing import Optional
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import BadgeResponse


class BaseBadges(BaseMoodle):
    def get_user_badges(self,
                        userid: Optional[int] = 0,
                        courseid: Optional[int] = 0,
                        page: Optional[int] = 0,
                        perpage: Optional[int] = 0,
                        search: Optional[str] = None,
                        onlypublic: Optional[int] = None) -> BadgeResponse:
        """Get list of badges awarded to a user.

        Args:
            userid (Optional[int], optional): Badges only for this user id, empty for current user. Defaults to 0.
            courseid (Optional[int], optional): Filter badges by course id, empty all the courses. Defaults to 0.
            page (Optional[int], optional): The page of records to return.. Defaults to 0.
            perpage (Optional[int], optional): The number of records to return per page. Defaults to 0.
            search (Optional[str], optional): A simple string to search for. Defaults to None.
            onlypublic (Optional[int], optional): Whether to return only public badges. Defaults to None.

        Returns:
            BadgeResponse: Returns the list of badges awarded to a user
        """
        res = self.moodle.post(
            'core_badges_get_user_badges',
            userid=userid,
            courseid=courseid,
            page=page,
            perpage=perpage,
            search=search,
            onlypublic=onlypublic,
        )
        return from_dict(BadgeResponse, res)



from typing import List, Optional
from moodle import BaseMoodle, Array
from moodle.utils.helper import from_dict
from . import (ActivityOverview, Course, CourseByField, SearchResult,
               CoursesBTC, CheckUpdate, Category, ContentOption, Section,
               CourseModule, NavigationOptions, ViewCourse)


class BaseCourse(BaseMoodle):
    def check_updates(self,
                      courseid: int,
                      tocheck: List[CheckUpdate.ToCheck],
                      filter: Optional[List[str]] = None) -> CheckUpdate:
        """Check if there is updates affecting the user for the given course and contexts.

        Args:
            courseid (int): Course id to check
            tocheck (List[CourseToCheck]): Instances to check
            filter (List[str], optional): Check only for updates in these areas. Defaults to [].

        Returns:
            CheckUpdate: Update
        """
        res = self.moodle.post("core_course_check_updates")
        return from_dict(CheckUpdate, res)

    def create_categories(self):
        res = self.moodle.post("core_course_create_categories")
        return res

    def create_courses(self):
        res = self.moodle.post("core_course_create_courses")
        return res

    def delete_categories(self):
        res = self.moodle.post("core_course_delete_categories")
        return res

    def delete_courses(self):
        res = self.moodle.post("core_course_delete_courses")
        return res

    def delete_modules(self):
        res = self.moodle.post("core_course_delete_modules")
        return res

    def duplicate_course(self):
        res = self.moodle.post("core_course_duplicate_course")
        return res

    def edit_module(self):
        res = self.moodle.post("core_course_edit_module")
        return res

    def edit_section(self):
        res = self.moodle.post("core_course_edit_section")
        return res

    def get_activities_overview(
            self,
            courseid: int,
            tocheck: List[Course.ToCheck],
            filter: Optional[List[str]] = None) -> ActivityOverview:
        res = self.moodle.post(
            "core_course_get_activities_overview",
            courseid=courseid,
            tocheck=tocheck,
            filter=filter or [],
        )
        return from_dict(ActivityOverview, res)

    def get_categories(
            self,
            criteria: Optional[List[Category.Criteria]] = None
    ) -> List[Category]:
        res = self.moodle.post(
            "core_course_get_categories",
            criteria=criteria or [],
        )
        return [from_dict(Category, data) for data in res] if res else []

    def get_contents(
            self,
            courseid: int,
            options: Optional[List[ContentOption]] = None) -> List[Section]:
        """Get course contents

        Args:
            courseid (int): course id
            options (Optional[List[ContentOption]], optional): Options, used since Moodle 2.9. Defaults to None.

        Returns:
            List[Section]: list of section
        """
        res = self.moodle.post("core_course_get_contents",
                               courseid=courseid,
                               options=options or [])
        return [from_dict(Section, data) for data in res] if res else []

    def get_course_module(self, cmid: int) -> CourseModule:
        """Return information about a course module

        Args:
            cmid (int): The course module id

        Returns:
            CourseModule: Course Module wrapper
        """
        res = self.moodle.post("core_course_get_course_module", cmid=cmid)
        return from_dict(CourseModule, res)

    def get_course_module_by_instance(self):
        res = self.moodle.post("core_course_get_course_module_by_instance")
        return res

    def get_courses(self, ids: Optional[List[int]] = None) -> List[Course]:
        """Get course details

        Args:
            ids (List[int], optional): List of course id. If empty return all courses except front page course. Defaults to None.

        Returns:
            List[Course]: Return course details
        """
        options = Array(ids if ids else [])
        options.name = 'ids'
        res = self.moodle.post("core_course_get_courses", options=options)
        return [from_dict(Course, data) for data in res] if res else []

    def get_courses_by_field(self,
                             field: str = '',
                             value: str = '') -> CourseByField:
        """Get courses matching a specific field (id/s, shortname, idnumber, category)

        Args:
            field (str, optional): The field to search can be left empty for all courses or:
                                   id: course id
                                   ids: comma separated course ids
                                   shortname: course short name
                                   idnumber: course id number
                                   category: category id the course belongs to.
                                   Defaults to ''.
            value (str, optional): The value to match. Defaults to ''.

        Returns:
            CourseByField: List of Course
        """
        res = self.moodle.post("core_course_get_courses_by_field",
                               field=field,
                               value=value)
        return from_dict(CourseByField, res)

    def get_enrolled_courses_by_timeline_classification(
            self,
            classification: str,
            limit: int = 0,
            offset: int = 0,
            sort: Optional[str] = None) -> CoursesBTC:
        """List of enrolled courses for the given timeline classification (past, inprogress, or future).

        Args:
            classification (str): future, inprogress, or past
            limit (int, optional): Result set limit. Defaults to 0.
            offset (int, optional): Result set offset. Defaults to 0.
            sort (Optional[str], optional): Sort string. Defaults to None.

        Returns:
            CoursesBTC: List of enrolled courses
        """
        res = self.moodle.post(
            "core_course_get_enrolled_courses_by_timeline_classification",
            classification=classification,
            limit=limit,
            offset=offset,
            sort=sort,
        )
        return from_dict(CoursesBTC, res)

    def get_recent_courses(self,
                           userid: int = 0,
                           limit: int = 0,
                           offset: int = 0,
                           sort: Optional[str] = None) -> List[Course]:
        res = self.moodle.post(
            "core_course_get_recent_courses",
            userid=userid,
            limit=limit,
            offset=offset,
            sort=sort,
        )
        return [from_dict(Course, data) for data in res] if res else []

    def get_enrolled_users_by_cmid(self):
        res = self.moodle.post("core_course_get_enrolled_users_by_cmid")
        return res

    def get_module(self):
        res = self.moodle.post("core_course_get_module")
        return res

    def get_updates_since(self,
                          courseid: int,
                          since: int,
                          filter: Optional[List[str]] = None) -> CheckUpdate:
        """Check if there are updates affecting the user for the given course since the given time stamp.

        Args:
            courseid (int): Course id to check
            since (int): Check updates since this time stamp
            filter (Optional[List[str]], optional): Check only for updates in these areas. Defaults to None.
                                                    Area name: configuration, fileareas, completion, ratings, comments, gradeitems, outcomes

        Returns:
            CheckUpdate: Update course detail
        """
        res = self.moodle.post("core_course_get_updates_since",
                               courseid=courseid,
                               since=since,
                               filter=filter)
        return from_dict(CheckUpdate, res)

    def get_user_administration_options(self):
        res = self.moodle.post("core_course_get_user_administration_options")
        return res

    def get_user_navigation_options(self,
                                    courseids: List[int]) -> NavigationOptions:
        """Return a list of navigation options in a set of courses that are avaialable or not for the current user.

        Args:
            courseids (List[int]): List of one or more course id

        Returns:
            NavigationOptions: Navigation options of courses
        """
        res = self.moodle.post("core_course_get_user_navigation_options",
                               courseids=courseids)
        return from_dict(NavigationOptions, res)

    def import_course(self):
        res = self.moodle.post("core_course_import_course")
        return res

    def search_courses(self,
                       criterianame: str,
                       criteriavalue: str,
                       page: int = 0,
                       perpage: int = 0,
                       requiredcapabilities: Optional[List[str]] = None,
                       limittoenrolled: int = 0,
                       onlywithcompletion: int = 0) -> SearchResult:
        """Search courses by (name, module, block, tag)

        Args:
            criterianame (str): criteria name (search, modulelist (only admins), blocklist (only admins), tagid)
            criteriavalue (str): criteria value
            page (int, optional): page number (0 based). Defaults to 0.
            perpage (int, optional): items per page. Defaults to 0.
            requiredcapabilities (Optional[List[str]], optional): Optional list of required capabilities (used to filter the list). Defaults to None.
            limittoenrolled (int, optional): limit to enrolled courses. Defaults to 0.
            onlywithcompletion (int, optional): limit to courses where completion is enabled. Defaults to 0.

        Returns:
            SearchResult: List of result courses
        """
        res = self.moodle.post(
            "core_course_search_courses",
            criterianame=criterianame,
            criteriavalue=criteriavalue,
            page=page,
            perpage=perpage,
            requiredcapabilities=requiredcapabilities,
            limittoenrolled=limittoenrolled,
            onlywithcompletion=onlywithcompletion,
        )
        return from_dict(SearchResult, res)

    def set_favourite_courses(self):
        res = self.moodle.post("core_course_set_favourite_courses")
        return res

    def update_categories(self):
        res = self.moodle.post("core_course_update_categories")
        return res

    def update_courses(self):
        res = self.moodle.post("core_course_update_courses")
        return res

    def view_course(self, courseid: int, sectionnumber: int) -> ViewCourse:
        res = self.moodle.post("core_course_view_course")
        return from_dict(ViewCourse, res)



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class ViewNotes:
    """Return type for view notes
    Constructor arguments:
    params: status (int): status: true if success
    params: warnings (List[Warning]): status: list of warnings
    """
    status: int
    warnings: List[Warning]



from datetime import datetime
from dataclasses import dataclass, field
from typing import List, Optional, Union
from moodle import ResponsesFactory


@dataclass
class CourseFormatOption:
    """Format Option
    Args:
        name (str): course format option name
        value (Union[str, int]): course format option value
    """
    name: str
    value: Union[str, int]


@dataclass
class CourseCustomField:
    """Custom Field
    Args:
        name (str): The name of the custom field
        shortname (str): The shortname of the custom field
        type (str): The type of the custom field - text, checkbox...
        value (Optional[str]): The value of the custom field
    """
    name: str
    shortname: str
    type: str
    value: Optional[str]


@dataclass
class Course:
    """Course
    Args:
        id (int): course id
        shortname (str): course short name
        categoryid (int): category id
        categorysortorder (Optional[int]): sort order into the category
        fullname (str): full name
        displayname (str): course display name
        idnumber (Optional[str]): id number
        summary (str): summary
        summaryformat (int): summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        format (str): course format: weeks, topics, social, site,..
        showgrades (Optional[int]): 1 if grades are shown, otherwise 0
        newsitems (Optional[int]): number of recent items appearing on the course page
        startdate (int): timestamp when the course start
        enddate (int): timestamp when the course end
        numsections (Optional[int]): (deprecated, use courseformatoptions) number of weeks/topics
        maxbytes (Optional[int]): largest size of file that can be uploaded into the course
        showreports (Optional[int]): are activity report shown (yes = 1, no =0)
        visible (Optional[int]): 1: available to student, 0:not available
        hiddensections (Optional[int]): (deprecated, use courseformatoptions) How the hidden sections in the course are displayed to students
        groupmode (Optional[int]): no group, separate, visible
        groupmodeforce (Optional[int]): 1: yes, 0: no
        defaultgroupingid (Optional[int]): default grouping id
        timecreated (Optional[int]): timestamp when the course have been created
        timemodified (Optional[int]): timestamp when the course have been modified
        enablecompletion (Optional[int]): Enabled, control via completion and activity settings. Disbaled, not shown in activity settings.
        completionnotify (Optional[int]): 1: yes 0: no
        lang (Optional[str]): forced course language
        forcetheme (Optional[str]): name of the force theme
        courseformatoptions (List[CourseFormatOption]): additional options for particular course format
        customfields (List[CourseCustomField]): Custom fields and associated values
    """
    id: int
    shortname: str
    categoryid: int
    categorysortorder: Optional[int]
    fullname: str
    displayname: str
    idnumber: Optional[str]
    summary: str
    summaryformat: int
    format: str
    showgrades: Optional[int]
    newsitems: Optional[int]
    startdate: int
    enddate: int
    numsections: Optional[int]
    maxbytes: Optional[int]
    showreports: Optional[int]
    visible: Optional[int]
    hiddensections: Optional[int]
    groupmode: Optional[int]
    groupmodeforce: Optional[int]
    defaultgroupingid: Optional[int]
    timecreated: Optional[int]
    timemodified: Optional[int]
    enablecompletion: Optional[int]
    completionnotify: Optional[int]
    lang: Optional[str]
    forcetheme: Optional[str]
    courseformatoptions: List[CourseFormatOption]
    customfields: List[CourseCustomField] = field(default_factory=list)

    @dataclass
    class ToCheck:
        """Course To Check arg
        Args:
            contextlevel (str): The context level for the file location. Only module supported right now.
            id (int): Context instance id
            since (datetime): Check updates since this time stamp
        """
        contextlevel: str
        id: int
        since: datetime


@dataclass
class CourseByField(ResponsesFactory):
    """Get Course By Field
    Args:
        courses (List[Course]): Course
        warnings (List[Warning]): list of warnings
    """
    courses: List[Course]
    warnings: List[Warning]

    @property
    def items(self) -> List[Course]:
        return self.courses


@dataclass
class SearchResult(ResponsesFactory[Course]):
    """Search Result
    Args:
        total (int): total course count
        courses (List[Course]): course
        warnings (List[Warning]): list of warning
    """
    total: int
    courses: List[Course]
    warnings: List[Warning]

    @property
    def items(self) -> List[Course]:
        return self.courses


@dataclass
class CourseBTC:
    """Course By Time Classification
    Args:
        id (int): id
        fullname (str): fullname
        shortname (str): shortname
        idnumber (str): idnumber
        summary (str): summary
        summaryformat (int): summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        startdate (int): startdate
        enddate (int): enddate
        visible (int): visible
        fullnamedisplay (str): fullnamedisplay
        viewurl (str): viewurl
        courseimage (str): courseimage
        progress (Optional[int]): progress
        hasprogress (int): hasprogress
        isfavourite (int): isfavourite
        hidden (int): hidden
        timeaccess (Optional[int]): timeaccess
        showshortname (int): showshortname
        coursecategory (str): coursecategory
    """
    id: int
    fullname: str
    shortname: str
    idnumber: str
    summary: str
    summaryformat: int
    startdate: int
    enddate: int
    visible: int
    fullnamedisplay: str
    viewurl: str
    courseimage: str
    progress: Optional[int]
    hasprogress: int
    isfavourite: int
    hidden: int
    timeaccess: Optional[int]
    showshortname: int
    coursecategory: str


@dataclass
class CoursesBTC(ResponsesFactory[CourseBTC]):
    courses: List[CourseBTC]
    nextoffset: int

    @property
    def items(self) -> List[CourseBTC]:
        return self.courses



from .config import MobileConfig
from .content import Restrict, File, OtherData, Template, Content
from .function import FunctionsResponses
from .key import Key
from .plugin import MobilePlugin
from .public_config import MobilePublicConfig

from .base import BaseMobile

__all__ = [
    'MobileConfig',
    'Restrict',
    'File',
    'OtherData',
    'Template',
    'Content',
    'FunctionsResponses',
    'Key',
    'MobilePlugin',
    'MobilePublicConfig',
    'BaseMobile',
]



from dataclasses import dataclass
from typing import Optional, Union


@dataclass
class Category:
    """Category

    Constructor arguments:
    params: id (int): category id
    params: name (str): category name
    params: idnumber (Optional[str]): category id number
    params: description (str): category description
    params: descriptionformat (int): description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: parent (int): parent category id
    params: sortorder (int): category sorting order
    params: coursecount (int): number of courses in this category
    params: visible (Optional[int]): 1: available, 0:not available
    params: visibleold (Optional[int]): 1: available, 0:not available
    params: timemodified (Optional[int]): timestamp
    params: depth (int): category depth
    params: path (str): category path
    params: theme (Optional[str]): category theme

    Returns:
        Category: Category
    """
    id: int
    name: str
    idnumber: Optional[str]
    description: str
    descriptionformat: int
    parent: int
    sortorder: int
    coursecount: int
    visible: Optional[int]
    visibleold: Optional[int]
    timemodified: Optional[int]
    depth: int
    path: str
    theme: Optional[str]

    def __str__(self) -> str:
        return self.name

    @staticmethod
    @dataclass
    class Criteria:
        """Criteria
        Constructor arguments::
        :param key (str): The category column to search,
            expected keys (value format) are:"id" (int) the category id,
            "ids" (string) category ids separated by commas,
            "name" (string) the category name,
            "parent" (int) the parent category id,
            "idnumber" (string) category idnumber - user must have 'moodle/category:manage' to search on idnumber,
            "visible" (int) whether the returned categories must be visible or hidden.

            If the key is not passed, then the function return all categories that the user can see.
            - user must have 'moodle/category:manage' or 'moodle/category:viewhiddencategories' to search on visible,
            "theme" (string) only return the categories having this theme - user must have 'moodle/category:manage' to search on theme
        Returns:
            Criteria: Arg for core_course_get_categories
        """
        key: str
        value: Union[str, int]



from .access_information import AccessInformation
from .allowed_event_types import AllowedEventTypes
from .event import Event, Events
from .course_event import (CourseEventIcon, CourseEventCategory,
                           CourseEventCourse, CourseEventSubscription,
                           CourseEventAction, CourseEvent, CourseEvents,
                           ActionEventCourses)
from .period import Period
from .date_view import DateView
from .day_view import DayView
from .monthly_view import Day, Week, DayName, MonthlyView

from .base import BaseCalendar

__all__ = [
    'AccessInformation', 'AllowedEventTypes', 'Event', 'Events',
    'CourseEventIcon', 'CourseEventCategory', 'CourseEventCourse',
    'CourseEventSubscription', 'CourseEventAction', 'CourseEvent',
    'CourseEvents', 'ActionEventCourses', 'Period', 'DateView', 'DayView',
    'Day', 'Week', 'DayName', 'MonthlyView', 'BaseCalendar'
]



from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning


@dataclass
class AccessInformation:
    """Forum Access Information
    Args:
        warnings (List[Warning]): list of warnings
        canaddinstance (Optional[int]): Whether the user has the capability mod/forum:addinstance allowed.
        canviewdiscussion (Optional[int]): Whether the user has the capability mod/forum:viewdiscussion allowed.
        canviewhiddentimedposts (Optional[int]): Whether the user has the capability mod/forum:viewhiddentimedposts allowed.
        canstartdiscussion (Optional[int]): Whether the user has the capability mod/forum:startdiscussion allowed.
        canreplypost (Optional[int]): Whether the user has the capability mod/forum:replypost allowed.
        canaddnews (Optional[int]): Whether the user has the capability mod/forum:addnews allowed.
        canreplynews (Optional[int]): Whether the user has the capability mod/forum:replynews allowed.
        canviewrating (Optional[int]): Whether the user has the capability mod/forum:viewrating allowed.
        canviewanyrating (Optional[int]): Whether the user has the capability mod/forum:viewanyrating allowed.
        canviewallratings (Optional[int]): Whether the user has the capability mod/forum:viewallratings allowed.
        canrate (Optional[int]): Whether the user has the capability mod/forum:rate allowed.
        canpostprivatereply (Optional[int]): Whether the user has the capability mod/forum:postprivatereply allowed.
        canreadprivatereplies (Optional[int]): Whether the user has the capability mod/forum:readprivatereplies allowed.
        cancreateattachment (Optional[int]): Whether the user has the capability mod/forum:createattachment allowed.
        candeleteownpost (Optional[int]): Whether the user has the capability mod/forum:deleteownpost allowed.
        candeleteanypost (Optional[int]): Whether the user has the capability mod/forum:deleteanypost allowed.
        cansplitdiscussions (Optional[int]): Whether the user has the capability mod/forum:splitdiscussions allowed.
        canmovediscussions (Optional[int]): Whether the user has the capability mod/forum:movediscussions allowed.
        canpindiscussions (Optional[int]): Whether the user has the capability mod/forum:pindiscussions allowed.
        caneditanypost (Optional[int]): Whether the user has the capability mod/forum:editanypost allowed.
        canviewqandawithoutposting (Optional[int]): Whether the user has the capability mod/forum:viewqandawithoutposting allowed.
        canviewsubscribers (Optional[int]): Whether the user has the capability mod/forum:viewsubscribers allowed.
        canmanagesubscriptions (Optional[int]): Whether the user has the capability mod/forum:managesubscriptions allowed.
        canpostwithoutthrottling (Optional[int]): Whether the user has the capability mod/forum:postwithoutthrottling allowed.
        canexportdiscussion (Optional[int]): Whether the user has the capability mod/forum:exportdiscussion allowed.
        canexportpost (Optional[int]): Whether the user has the capability mod/forum:exportpost allowed.
        canexportownpost (Optional[int]): Whether the user has the capability mod/forum:exportownpost allowed.
        canaddquestion (Optional[int]): Whether the user has the capability mod/forum:addquestion allowed.
        canallowforcesubscribe (Optional[int]): Whether the user has the capability mod/forum:allowforcesubscribe allowed.
        cancanposttomygroups (Optional[int]): Whether the user has the capability mod/forum:canposttomygroups allowed.
        cancanoverridediscussionlock (Optional[int]): Whether the user has the capability mod/forum:canoverridediscussionlock allowed.
        cancanoverridecutoff (Optional[int]): Whether the user has the capability mod/forum:canoverridecutoff allowed.
        cancantogglefavourite (Optional[int]): Whether the user has the capability mod/forum:cantogglefavourite allowed.
    """
    warnings: List[Warning]
    canaddinstance: Optional[int]
    canviewdiscussion: Optional[int]
    canviewhiddentimedposts: Optional[int]
    canstartdiscussion: Optional[int]
    canreplypost: Optional[int]
    canaddnews: Optional[int]
    canreplynews: Optional[int]
    canviewrating: Optional[int]
    canviewanyrating: Optional[int]
    canviewallratings: Optional[int]
    canrate: Optional[int]
    canpostprivatereply: Optional[int]
    canreadprivatereplies: Optional[int]
    cancreateattachment: Optional[int]
    candeleteownpost: Optional[int]
    candeleteanypost: Optional[int]
    cansplitdiscussions: Optional[int]
    canmovediscussions: Optional[int]
    canpindiscussions: Optional[int]
    caneditanypost: Optional[int]
    canviewqandawithoutposting: Optional[int]
    canviewsubscribers: Optional[int]
    canmanagesubscriptions: Optional[int]
    canpostwithoutthrottling: Optional[int]
    canexportdiscussion: Optional[int]
    canexportpost: Optional[int]
    canexportownpost: Optional[int]
    canaddquestion: Optional[int]
    canallowforcesubscribe: Optional[int]
    cancanposttomygroups: Optional[int]
    cancanoverridediscussionlock: Optional[int]
    cancanoverridecutoff: Optional[int]
    cancantogglefavourite: Optional[int]



from typing import List
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import Resources, View


class BaseResource(BaseMoodle):
    def get_resources_by_courses(self, courseids: List[int]) -> Resources:
        """Returns a list of files in a provided list of courses, if no list is provided all files that the user can view will be returned.

        Args:
            courseids (List[int]): Array of course ids

        Returns:
            Resources: List of Resource
        """
        res = self.moodle.post('mod_resource_get_resources_by_courses',
                               courseids=courseids)
        return from_dict(Resources, res)

    def view_resource(self, resourceid: int) -> View:
        """Simulate the view.php web interface resource: trigger events, completion, etc...

        Args:
            resourceid (int): resource instance id

        Returns:
            View: View Resource response
        """
        res = self.moodle.post('mod_resource_view_resource',
                               resourceid=resourceid)
        return from_dict(View, res)



from datetime import datetime
from typing import List, Optional, Union
from moodle import BaseMoodle
from moodle.utils.helper import from_dict
from . import AccessInformation, Discussions, Forum, NewPost, Posts


class BaseForum(BaseMoodle):
    def add_discussion(
            self,
            forumid: int,
            subject: str,
            message: str,
            groupid: int = 0,
            options: Optional[List[Discussions.Option]] = None
    ) -> Discussions.New:
        """Add a new discussion into an existing forum.

        Args:
            forumid (int): Forum instance ID
            subject (str): New Discussion subject
            message (str): New Discussion message (only html format allowed)
            groupid (int, optional): The group, default to 0. Defaults to 0.
            options (List[Discussions.Option], optional): Options. Defaults to None.

        Returns:
            Discussions.New: [description]
        """
        res = self.moodle.post(
            'mod_forum_add_discussion',
            forumid=forumid,
            subject=subject,
            message=message,
            groupid=groupid,
            options=options or [],
        )
        return from_dict(Discussions.New, res)

    def add_discussion_post(self,
                            postid: int,
                            subject: str,
                            message: str,
                            options: Optional[List[Posts.Option]] = None,
                            messageformat: int = 1) -> NewPost:
        """Create new posts into an existing discussion.

        Args:
            postid (int): the post id we are going to reply to (can be the initial discussion post
            subject (str): new post subject
            message (str): new post message (html assumed if messageformat is not provided)
            options (List[Posts.Option], optional): Options. Defaults to None.
            messageformat (int, optional): message format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN). Defaults to 1.

        Returns:
            NewPost: [description]
        """
        res = self.moodle.post(
            'mod_forum_add_discussion_post',
            postid=postid,
            subject=subject,
            message=message,
            options=options or [],
            messageformat=messageformat,
        )
        return from_dict(NewPost, res)

    def can_add_discussion(
            self,
            forumid: int,
            groupid: Optional[int] = None) -> Discussions.CanAdd:
        res = self.moodle.post(
            'mod_forum_can_add_discussion',
            forumid=forumid,
            groupid=groupid,
        )
        return from_dict(Discussions.CanAdd, res)

    def get_discussion_posts(self,
                             discussionid: int,
                             sortby: str = 'created',
                             sortdirection: str = 'DESC') -> Posts:
        """Returns a list of forum posts for a discussion.

        Args:
            discussionid (int): The ID of the discussion from which to fetch posts.
            sortby (str, optional): Sort by this element: id, created or modified. Defaults to 'created'.
            sortdirection (str, optional): Sort direction: ASC or DESC. Defaults to 'DESC'.

        Returns:
            Posts: List of Post
        """
        res = self.moodle.post(
            'mod_forum_get_discussion_posts',
            discussionid=discussionid,
            sortby=sortby,
            sortdirection=sortdirection,
        )
        return from_dict(Posts, res)

    def get_forum_access_information(self, forumid: int) -> AccessInformation:
        """Return capabilities information for a given forum.

        Args:
            forumid (int): Forum instance id.

        Returns:
            AccessInformation: Access Information of Forum
        """
        res = self.moodle.post(
            'mod_forum_get_forum_access_information',
            forumid=forumid,
        )
        return from_dict(AccessInformation, res)

    def get_forum_discussion_posts(self,
                                   discussionid: int,
                                   sortby: str = 'created',
                                   sortdirection: str = 'DESC') -> Posts:
        """Returns a list of forum posts for a discussion.

        Args:
            discussionid (int): discussion ID
            sortby (str, optional): sort by this element: id, created or modified. Defaults to 'created'.
            sortdirection (str, optional): sort direction: ASC or DESC. Defaults to 'DESC'.

        Returns:
            Posts: List of Post
        """
        res = self.moodle.post(
            'mod_forum_get_forum_discussion_posts',
            discussionid=discussionid,
            sortby=sortby,
            sortdirection=sortdirection,
        )
        return from_dict(Posts, res)

    def get_forum_discussions(self,
                              forumid: int,
                              sortorder: int = -1,
                              page: int = -1,
                              perpage: int = 0,
                              groupid: int = 0) -> Discussions:
        """Returns a list of forum discussions optionally sorted and paginated.

        Args:
            forumid (int): forum instance id
            sortorder (int, optional): sort by this element: numreplies, , created or timemodified. Defaults to -1.
            page (int, optional): current page. Defaults to -1.
            perpage (int, optional): items per page. Defaults to 0.
            groupid (int, optional): group id. Defaults to 0.

        Returns:
            Discussions: List of Discussion
        """
        res = self.moodle.post(
            'mod_forum_get_forum_discussions',
            forumid=forumid,
            sortorder=sortorder,
            page=page,
            perpage=perpage,
            groupid=groupid,
        )
        return from_dict(Discussions, res)

    def get_forum_discussions_paginated(self,
                                        forumid: int,
                                        sortby: str = 'timemodified',
                                        sortdirection: str = 'DESC',
                                        page: int = -1,
                                        perpage: int = 0) -> Discussions:
        """** DEPRECATED ** Please do not call this function any more. Returns a list of forum discussions optionally sorted and paginated.

        Args:
            forumid (int): forum instance id
            sortby (str, optional): sort by this element: id, timemodified, timestart or timeend. Defaults to 'timemodified'.
            sortdirection (str, optional): sort direction: ASC or DESC. Defaults to 'DESC'.
            page (int, optional): current page. Defaults to -1.
            perpage (int, optional): items per page. Defaults to 0.

        Returns:
            Discussions: list of Discussion
        """
        res = self.moodle.post(
            'mod_forum_get_forum_discussions_paginated',
            forumid=forumid,
            sortby=sortby,
            sortdirection=sortdirection,
            page=page,
            perpage=perpage,
        )
        return from_dict(Discussions, res)

    def get_forums_by_courses(self,
                              courseids: Optional[List[int]] = None
                              ) -> List[Forum]:
        """Returns a list of forum instances in a provided set of courses, if no courses are provided then all the forum instances the user has access to will be returned.

        Args:
            courseids (Optional[List[int]], optional): Array of Course IDs. Defaults to None.

        Returns:
            List[Forum]: list of Forum
        """
        datas = self.moodle.post('mod_forum_get_forums_by_courses',
                                 courseids=courseids or [])
        return [Forum.from_data(data, self.moodle)
                for data in datas] if datas else []

    def set_lock_state(self, forumid: int, discussionid: int,
                       targetstate: Union[datetime, int]):
        res = self.moodle.post(
            'mod_forum_set_lock_state',
            forumid=forumid,
            discussionid=discussionid,
            targetstate=targetstate,
        )
        return res

    def set_pin_state(self, discussionid: int, targetstate: int):
        res = self.moodle.post(
            'mod_forum_set_pin_state',
            discussionid=discussionid,
            targetstate=targetstate,
        )
        return res

    def set_subscription_state(self, forumid: int, discussionid: int,
                               targetstate: int):
        res = self.moodle.post(
            'mod_forum_set_subscription_state',
            forumid=forumid,
            discussionid=discussionid,
            targetstate=targetstate,
        )
        return res

    def toggle_favourite_state(self, discussionid: int, targetstate: int):
        res = self.moodle.post(
            'mod_forum_toggle_favourite_state',
            discussionid=discussionid,
            targetstate=targetstate,
        )
        return res

    def view_forum(self, forumid: int):
        res = self.moodle.post(
            'mod_forum_view_forum',
            forumid=forumid,
        )
        return res

    def view_forum_discussion(self, discussionid: int):
        res = self.moodle.post(
            'mod_forum_view_forum_discussion',
            discussionid=discussionid,
        )
        return res



from dataclasses import dataclass, field
from typing import List, Optional
from moodle import ResponsesFactory


@dataclass
class Response:
    """Response when calling external function
    Constructor arguments:
    params: error (int): Whether an exception was thrown.
    params: data (str): JSON-encoded response data
    params: exception (str): JSON-encoed exception info
    """
    error: int
    data: str
    exception: str


@dataclass
class FunctionsResponses(ResponsesFactory[Response]):
    responses: List[Response] = field(default_factory=list)

    @property
    def items(self) -> List[Response]:
        return self.responses

    @dataclass
    class Request:
        """For requesting external function
        Constructor arguments:
        params: function (str): Function name
        params: arguments (str): Default for "{}" //JSON-encoded object with named arguments
        params: settingraw (int): Default for "" //Return raw text
        params: settingfilter (int): Default for "" //Filter text
        params: settingfileurl (int): Default for "1" //Rewrite plugin file URLs
        params: settinglang (str): Default for "" //Session language
        """
        function: str
        arguments: str = "{}"
        settingraw: Optional[int] = None
        settingfilter: Optional[int] = None
        settingfileurl: int = 1
        settinglang: str = ""



from dacite import from_dict as dacite_from_dict
from dacite.data import Data
from dataclasses import is_dataclass, asdict
from datetime import datetime
from typing import Type, TypeVar, Any

from moodle.config import DACITE_CONFIG

T = TypeVar("T")


def make_params(wstoken: str,
                wsfunction: str,
                moodlewsrestformat: str = 'json') -> dict:
    return {
        'wstoken': wstoken,
        'wsfunction': wsfunction,
        'moodlewsrestformat': moodlewsrestformat
    }


def from_dict(data_class: Type[T], data: Data) -> T:
    return dacite_from_dict(data_class, data, config=DACITE_CONFIG)


def to_dict(data: Any, name: str = '') -> Any:
    if not data:
        return data
    if isinstance(data, list):
        out = {}
        for idx, val in enumerate(data):
            val = to_dict(val)
            if isinstance(val, dict):
                for key, value in val.items():
                    out[f'{name}[{idx}][{key}]'] = val[key]
            else:
                out_key = name
                # Check if data required name prefix
                if hasattr(data, 'name'):
                    out_key += f"[{getattr(data, 'name')}]"
                out_key += f'[{idx}]'
                out[out_key] = val
        return out
    if isinstance(data, dict):
        out = {}
        for key, value in data.items():
            if isinstance(value, list):
                out.update(to_dict(value, key))
            else:
                out[key] = value
        return out
    if is_dataclass(data):
        return asdict(data)
    if isinstance(data, datetime):
        return datetime.timestamp(data)
    return data



from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning, ResponsesFactory
from . import SiteNote, CourseNote, PersonalNote


@dataclass
class CourseNotes(ResponsesFactory[CourseNote]):
    """Course Notes
    Constructor arguments:
    params: sitenotes (List[SiteNote]): site notes
    params: coursenotes (List[CourseNote]): couse notes
    params: personalnotes (List[PersonalNote]): personal notes
    params: canmanagesystemnotes (Optional[int]): Whether the user can manage notes at system level.
    params: canmanagecoursenote (Optional[int]): Whether the user can manage notes at the given course.
    params: warnings (List[Warning]): list of warnings
    """
    sitenotes: List[SiteNote]
    coursenotes: List[CourseNote]
    personalnotes: List[PersonalNote]
    canmanagesystemnotes: Optional[int]
    canmanagecoursenote: Optional[int]
    warnings: List[Warning]

    @property
    def items(self) -> List[CourseNote]:
        return self.coursenotes



from .base.moodle_object import MoodleObject
from .base.name_value import BaseNameValue
from .base.base_moodle import BaseMoodle
from .base.warning import Warning
from .base.responses import ResponsesFactory
from .base.preference import (MessagePreference, NotificationPreference,
                              UserPreference)

from .exception import MoodleException
from .utils.typing import Array

from .tool import Tool
from .auth import Auth
from .core import Core
from .mod import Mod

from .mdl import Mdl
from .mdl import Moodle

from .version import __version__  # NOQA

__all__ = [
    'MoodleObject',
    'BaseNameValue',
    'BaseMoodle',
    'Warning',
    'ResponsesFactory',
    'MessagePreference',
    'NotificationPreference',
    'UserPreference',
    'MoodleException',
    'Array',
    'Auth',
    'Core',
    'Mod',
    'Tool',
    'Mdl',
    'Moodle',
]



from .access_information import AccessInformation
from .discussion import Discussion, Discussions
from .forum import Forum
from .post import (AuthorUrls, GroupUrls, Group, PlagiarismHtml,
                   AttachmentUrls, TagUrls, RatingInfo, Html, Tag, Attachment,
                   PostUrls, Capability, Author, Post, Message, NewPost, Posts)

from .base import BaseForum

__all__ = [
    'AccessInformation', 'Discussion', 'Discussions', 'Forum', 'AuthorUrls',
    'GroupUrls', 'Group', 'PlagiarismHtml', 'AttachmentUrls', 'TagUrls',
    'RatingInfo', 'Html', 'Tag', 'Attachment', 'PostUrls', 'Capability',
    'Author', 'Post', 'Message', 'NewPost', 'Posts', 'BaseForum'
]



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class ViewCourse:
    status: int  # status: true if success
    warnings: List[Warning]  # list of warnings



from .activity_overview import ActivityOverview
from .check_updates import CheckUpdate
from .content_option import ContentOption

from .course import Course, CourseByField, SearchResult
from .course import CourseBTC, CoursesBTC
from .category import Category
from .content import (
    ContentInfo,
    Tag,
    Content,
    Completion,
    Module,
    Section,
)
from .course_module import CourseModule
from .navigation_option import NavigationOptions
from .view import ViewCourse

from .base import BaseCourse

__all__ = [
    'ActivityOverview', 'CheckUpdate', 'ContentOption', 'Course',
    'CourseByField', 'SearchResult', 'CourseBTC', 'CoursesBTC', 'Category',
    'ContentInfo', 'Tag', 'Content', 'Completion', 'Module', 'Section',
    'CourseModule', 'NavigationOptions', 'ViewCourse', 'BaseCourse'
]



from moodle import BaseMoodle
from . import (BaseBadges, BaseBlock, BaseBlog, BaseCalendar, BaseCohort,
               BaseComment, BaseCompetency, BaseCompletion, BaseCourse,
               BaseMessage, BaseNotes, BaseUser, BaseWebservice)


class Core(BaseMoodle):
    def __post_init__(self, moodle) -> None:
        self._badges = BaseBadges(moodle)
        self._block = BaseBlock(moodle)
        self._blog = BaseBlog(moodle)
        self._calendar = BaseCalendar(moodle)
        self._cohort = BaseCohort(moodle)
        self._comment = BaseComment(moodle)
        self._competency = BaseCompetency(moodle)
        self._completion = BaseCompletion(moodle)
        self._course = BaseCourse(moodle)
        self._message = BaseMessage(moodle)
        self._notes = BaseNotes(moodle)
        self._user = BaseUser(moodle)
        self._webservice = BaseWebservice(moodle)

    @property
    def bagdes(self) -> BaseBadges:
        return self._badges

    @property
    def block(self) -> BaseBlock:
        return self._block

    @property
    def blog(self) -> BaseBlog:
        return self._blog

    @property
    def calendar(self) -> BaseCalendar:
        return self._calendar

    @property
    def cohort(self) -> BaseCohort:
        return self._cohort

    @property
    def comment(self) -> BaseComment:
        return self._comment

    @property
    def competency(self) -> BaseCompetency:
        return self._competency

    @property
    def completion(self) -> BaseCompletion:
        return self._completion

    @property
    def course(self) -> BaseCourse:
        return self._course

    @property
    def message(self) -> BaseMessage:
        return self._message

    @property
    def notes(self) -> BaseNotes:
        return self._notes

    @property
    def user(self) -> BaseUser:
        return self._user

    @property
    def webservice(self) -> BaseWebservice:
        return self._webservice



from datetime import datetime
from dataclasses import dataclass, field
from typing import List, Optional


@dataclass
class ContentInfo:
    """Content Info
    Args:
        filescount (int): Total number of files.
        filessize (int): Total files size.
        lastmodified (datetime): Last time files were modified.
        mimetypes (List[str]): Files mime types.
        repositorytype (Optional[str]): The repository type for the main file.
    """
    filescount: int
    filessize: int
    lastmodified: datetime
    mimetypes: List[str]
    repositorytype: Optional[str]


@dataclass
class Tag:
    """Tag
    Args
        id (int): Tag id.
        name (str): Tag name.
        rawname (str): The raw, unnormalised name for the tag as entered by users.
        isstandard (int): Whether this tag is standard.
        tagcollid (int): Tag collection id.
        taginstanceid (int): Tag instance id.
        taginstancecontextid (int): Context the tag instance belongs to.
        itemid (int): Id of the record tagged.
        ordering (int): Tag ordering.
        flag (int): Whether the tag is flagged as inappropriate.
    """
    id: int
    name: str
    rawname: str
    isstandard: int
    tagcollid: int
    taginstanceid: int
    taginstancecontextid: int
    itemid: int
    ordering: int
    flag: int


@dataclass
class Content:
    """Content
    Args:
        type (str): a file or a folder or external link
        filename (str): filename
        filepath (str): filepath
        filesize (int): filesize
        fileurl (Optional[str]): downloadable file url
        content (Optional[str]): Raw content, will be used when type is content
        timecreated (datetime): Time created
        timemodified (datetime): Time modified
        sortorder (int): Content sort order
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
        userid (int): User who added this content to moodle
        author (str): Content owner
        license (str): Content license
        tags (List[Tag]): Tags
    """
    type: str
    filename: str
    filepath: str
    filesize: int
    fileurl: Optional[str]
    content: Optional[str]
    timecreated: datetime
    timemodified: datetime
    sortorder: int
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]
    userid: int
    author: str
    license: str
    tags: List[Tag] = field(default_factory=list)


@dataclass
class Completion:
    """Completion
    Args:
        state (int): Completion state value: 0 means incomplete, 1 complete, 2 complete pass, 3 complete fail
        timecompleted (int): Timestamp for completion status.
        overrideby (int): The user id who has overriden the status.
        valueused (Optional[int]): Whether the completion status affects the availability of another activity.
    """
    state: int
    timecompleted: int
    overrideby: Optional[int]
    valueused: Optional[int]


@dataclass
class Module:
    """Module
    Args:
        id (int): activity id
        url (Optional[str]): activity url
        name (str): activity module name
        instance (Optional[int]): instance id
        description (Optional[str]): activity description
        visible (Optional[int]): is the module visible
        uservisible (Optional[int]): Is the module visible for the user?
        availabilityinfo (Optional[str]): Availability information.
        visibleoncoursepage (Optional[int]): is the module visible on course page
        modicon (str): activity icon url
        modname (str): activity module type
        modplural (str): activity module plural name
        availability (Optional[str]): module availability settings
        indent (int): number of identation in the site
        onclick (Optional[str]): Onclick action.
        afterlink (Optional[str]): After link info to be displayed.
        customdata (Optional[str]): Custom data (JSON encoded).
        completion (Optional[int]): Type of completion tracking: 0 means none, 1 manual, 2 automatic.
        completiondata (Optional[Completion]): Module completion data.
        contents (List[Content]): list of Content
        contentsinfo (Optional[ContentInfo]): Contents summary information.
    """
    id: int
    url: Optional[str]
    name: str
    instance: Optional[int]
    description: Optional[str]
    visible: Optional[int]
    uservisible: Optional[int]
    availabilityinfo: Optional[str]
    visibleoncoursepage: Optional[int]
    modicon: str
    modname: str
    modplural: str
    availability: Optional[str]
    indent: int
    onclick: Optional[str]
    afterlink: Optional[str]
    customdata: Optional[str]
    completion: Optional[int]
    completiondata: Optional[Completion]
    contents: List[Content] = field(default_factory=list)
    contentsinfo: Optional[ContentInfo] = None


@dataclass
class Section:
    """Section
    Args:
        id: (int): Section ID
        name: (str): Section name
        visible: (Optional[int]): is the section visible
        summary: (str): Section description
        summaryformat: (int): summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        section: (Optional[int]): Section number inside the course
        hiddenbynumsections: (Optional[int]): Whether is a section hidden in the course format
        uservisible: (Optional[int]): Is the section visible for the user?
        availabilityinfo: (Optional[str]): Availability information.
        modules: (List[Module]): list of module
    """
    id: int
    name: str
    visible: Optional[int]
    summary: str
    summaryformat: int
    section: Optional[int]
    hiddenbynumsections: Optional[int]
    uservisible: Optional[int]
    availabilityinfo: Optional[str]
    modules: List[Module]



from operator import getitem
from typing import Generic, Iterator, List, TypeVar

T = TypeVar('T')


class ResponsesFactory(Generic[T]):
    @property
    def items(self) -> List[T]:
        return []

    def __bool__(self) -> bool:
        return bool(self.items)

    def __getitem__(self, index: int) -> T:
        return getitem(self.items, index)

    def __iter__(self) -> Iterator[T]:
        return iter(self.items)

    def __len__(self) -> int:
        return len(self.items)

    def __str__(self) -> str:
        return str(self.items)

    def first(self) -> T:
        return getitem(self.items, 0)

    def last(self) -> T:
        return getitem(self.items, -1)



from __future__ import annotations
from dataclasses import dataclass
from typing import Optional, List
from moodle import ResponsesFactory


@dataclass
class CourseEventIcon:
    """Icon
    Constructor arguments:
    params: key (str): key
    params: component (str): component
    params: alttext (str): alttext
    """
    key: str
    component: str
    alttext: str


@dataclass
class CourseEventCategory:
    """Category
    Constructor arguments:
    params: id (int): id
    params: name (str): name
    params: idnumber (str): idnumber
    params: description (Optional[str]): description
    params: parent (int): parent
    params: coursecount (int): coursecount
    params: visible (int): visible
    params: timemodified (int): timemodified
    params: depth (int): depth
    params: nestedname (str): nestedname
    params: url (str): url
    """
    id: int
    name: str
    idnumber: str
    description: Optional[str]
    parent: int
    coursecount: int
    visible: int
    timemodified: int
    depth: int
    nestedname: str
    url: str

    def __str__(self) -> str:
        return self.name


@dataclass
class CourseEventCourse:
    """Course
    Constructor arguments:
    params: id (int): id
    params: fullname (str): fullname
    params: shortname (str): shortname
    params: idnumber (str): idnumber
    params: summary (str): summary
    params: summaryformat (int): summary format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: startdate (int): startdate
    params: enddate (int): enddate
    params: visible (int): visible
    params: fullnamedisplay (str): fullnamedisplay
    params: viewurl (str): viewurl
    params: courseimage (str): courseimage
    params: progress (Optional[int]): progress
    params: hasprogress (int): hasprogress
    params: isfavourite (int): isfavourite
    params: hidden (int): hidden
    params: timeaccess (Optional[int]): timeaccess
    params: showshortname (int): showshortname
    params: coursecategory (str): coursecategory
    """
    id: int
    fullname: str
    shortname: str
    idnumber: str
    summary: str
    summaryformat: int
    startdate: int
    enddate: int
    visible: int
    fullnamedisplay: str
    viewurl: str
    courseimage: str
    progress: Optional[int]
    hasprogress: int
    isfavourite: int
    hidden: int
    timeaccess: Optional[int]
    showshortname: int
    coursecategory: str

    def __str__(self) -> str:
        return self.fullname


@dataclass
class CourseEventSubscription:
    """Subscription
    Constructor arguments:
    params: displayeventsource (int): displayeventsource
    params: subscriptionname (Optional[str]): subscriptionname
    params: subscriptionurl (Optional[str]): subscriptionurl
    """
    displayeventsource: int
    subscriptionname: Optional[str]
    subscriptionurl: Optional[str]

    def __str__(self) -> str:
        return self.subscriptionname or repr(self)


@dataclass
class CourseEventAction:
    """Action
    Constructor arguments:
    params: name (str): name
    params: url (str): url
    params: itemcount (int): itemcount
    params: actionable (int): actionable
    params: showitemcount (int): showitemcount
    """
    name: str
    url: str
    itemcount: int
    actionable: int
    showitemcount: int

    def __str__(self) -> str:
        return self.name


@dataclass
class CourseEvent:
    """Course Event
    Constructor arguments:
    params: id (int): id
    params: name (str): name
    params: description (Optional[str]): description
    params: descriptionformat (int): Default untuk "1" description format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
    params: location (Optional[str]): location
    params: categoryid (Optional[int]): categoryid
    params: groupid (Optional[int]): groupid
    params: userid (Optional[int]): userid
    params: repeatid (Optional[int]): repeatid
    params: eventcount (Optional[int]): eventcount
    params: modulename (Optional[str]): modulename
    params: instance (Optional[int]): instance
    params: eventtype (str): eventtype
    params: timestart (int): timestart
    params: timeduration (int): timeduration
    params: timesort (int): timesort
    params: visible (int): visible
    params: timemodified (int): timemodified
    params: icon (CourseEventIcon)
    params: category (CourseEventCategory)
    params: course (CourseEventCourse)
    params: subscription (CourseEventSubscription)
    params: canedit (int): canedit
    params: candelete (int): candelete
    params: deleteurl (str): deleteurl
    params: editurl (str): editurl
    params: viewurl (str): viewurl
    params: formattedtime (str): formattedtime
    params: isactionevent (int): isactionevent
    params: iscourseevent (int): iscourseevent
    params: iscategoryevent (int): iscategoryevent
    params: groupname (Optional[str]): groupname
    params: normalisedeventtype (str): normalisedeventtype
    params: normalisedeventtypetext (str): normalisedeventtypetext
    params: url (str): url
    params: action (Optional[CourseEventAction])
    params: islastday (int): islastday
    params: popupname (str): popupname
    params: mindaytimestamp (Optional[int]): mindaytimestamp
    params: mindayerror (Optional[str]): mindayerror
    params: maxdaytimestamp (Optional[int]): maxdaytimestamp
    params: maxdayerror (Optional[str]): maxdayerror
    params: draggable (int): draggable
    """
    id: int
    name: str
    description: Optional[str]
    descriptionformat: int
    location: Optional[str]
    categoryid: Optional[int]
    groupid: Optional[int]
    userid: Optional[int]
    repeatid: Optional[int]
    eventcount: Optional[int]
    modulename: Optional[str]
    instance: Optional[int]
    eventtype: str
    timestart: int
    timeduration: int
    timesort: int
    visible: int
    timemodified: int
    icon: CourseEventIcon
    category: CourseEventCategory
    course: CourseEventCourse
    subscription: CourseEventSubscription
    canedit: int
    candelete: int
    deleteurl: str
    editurl: str
    viewurl: str
    formattedtime: str
    isactionevent: int
    iscourseevent: int
    iscategoryevent: int
    groupname: Optional[str]
    normalisedeventtype: str
    normalisedeventtypetext: str
    url: str
    action: Optional[CourseEventAction]
    islastday: int
    popupname: str
    mindaytimestamp: Optional[int]
    mindayerror: Optional[str]
    maxdaytimestamp: Optional[int]
    maxdayerror: Optional[str]
    draggable: int

    def __str__(self) -> str:
        return self.name


@dataclass
class CourseEvents(ResponsesFactory[CourseEvent]):
    """List of [CourseEvent]
    """
    events: List[CourseEvent]
    firstid: int
    lastid: int

    @property
    def items(self) -> List[CourseEvent]:
        return self.events

    @dataclass
    class EventForm:
        """Result submit form data for event form
        parms: event (CourseEvent): Resulted event
        parms: validationerror (Optional[int]): Default untuk "" Invalid form data
        """
        event: CourseEvent
        validationerror: Optional[int] = None


@dataclass
class ActionEventCourses(ResponsesFactory[CourseEvents]):
    """Calendar action events by courses

    Constructor arguments:
    params: groupedbycourse (List[CourseEvents]): Collection of course events

    Returns:
        ActionEventCourses: ActionEventCourses
    """
    groupedbycourse: List[CourseEvents]

    @property
    def items(self) -> List[CourseEvents]:
        return self.groupedbycourse



from dataclasses import dataclass
from typing import Optional


@dataclass
class UserFlag:
    """User Flag
    Args:
        userid (int): student id
        locked (Optional[int]): locked
        mailed (Optional[int]): mailed
        extensionduedate (Optional[int]): extension due date
        workflowstate (Optional[str]): marking workflow state
        allocatedmarker (Optional[int]): allocated marker
    """
    userid: int
    locked: Optional[int]
    mailed: Optional[int]
    extensionduedate: Optional[int]
    workflowstate: Optional[str]
    allocatedmarker: Optional[int]

    @dataclass
    class Result:
        """Result
        Args:
            id (int): id of record if successful, -1 for failure
            userid (int): userid of record
            errormessage (Optional[str]): Failure error message
        """
        id: int
        userid: int
        errormessage: Optional[str]



from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class ActivityCompletion:
    """Activity Completion
    Args:
        cmid (int): comment ID
        modname (str): activity module name
        instance (int): instance ID
        state (int): completion state value: 0 means incomplete, 1 complete, 2 complete pass, 3 complete fail
        timecompleted (int): timestamp for completed activity
        tracking (int): type of tracking: 0 means none, 1 manual, 2 automatic
        overrideby (Optional[int]): The user id who has overriden the status, or null
        valueused (Optional[int]): Whether the completion status affects the availability of another activity.
    """
    cmid: int
    modname: str
    instance: int
    state: int
    timecompleted: int
    tracking: int
    overrideby: Optional[int]
    valueused: Optional[int]


@dataclass
class ActivityCompletionStatus(ResponsesFactory[ActivityCompletion]):
    """Activity Completion Statuses (List of activities completion status)
    Args:
        statuses (List[Activity]): List of activities completion status
        warnings (List[Warning]): list of warnings
    """
    statuses: List[ActivityCompletion]
    warnings: List[Warning]

    @property
    def items(self) -> List[ActivityCompletion]:
        return self.statuses



from __future__ import annotations
import logging
import requests
from requests import Session
from typing import Any, Type, TypeVar
from moodle import Auth, Core, Mod, MoodleException, Tool, Warning
from moodle.exception import InvalidCredentialException
from moodle.utils.helper import make_params, from_dict, to_dict

T = TypeVar('T', bound='Mdl')


class Mdl:
    session: Session = Session()
    token: str = ''
    url: str = ''

    def __init__(self, url: str, token: str):
        self.logger = logging.getLogger(self.__class__.__name__)
        self.url = url
        self.token = token

    def get(self, wsfunction: str, moodlewsrestformat='json', **kwargs) -> Any:
        params = make_params(self.token, wsfunction, moodlewsrestformat)
        params.update(to_dict(kwargs))
        res = self.session.get(self.url, params=params)
        if res.ok and moodlewsrestformat == 'json':
            data = res.json()
            return self.process_response(data)
        return res.text

    def post(self,
             wsfunction: str,
             moodlewsrestformat='json',
             **kwargs: Any) -> Any:
        params = make_params(self.token, wsfunction, moodlewsrestformat)
        res = self.session.post(self.url, data=to_dict(kwargs), params=params)
        if res.ok and moodlewsrestformat == 'json':
            data = res.json()
            return self.process_response(data)
        return res.text

    def process_response(self, data: Any) -> Any:
        if type(data) == dict:
            if 'warnings' in data and data['warnings']:
                warning = from_dict(Warning, data['warnings'])
                self.logger.warning(str(warning))
            if 'exception' in data or 'errorcode' in data:
                raise from_dict(MoodleException, data)
        return data

    @classmethod
    def login(cls: Type[T],
              domain: str,
              username: str,
              password: str,
              service: str = 'moodle_mobile_app',
              loginurl: str = '/login/token.php',
              web_service_url: str = '/webservice/rest/server.php') -> T:
        """Get a Moodle instance by using username & password auth

        Args:
            domain (str): The domain of moodle app
            username (str): Username
            password (str): Password
            service (str, optional): Service name. Defaults to 'moodle_mobile_app'.
            loginurl (str, optional): Url to get token from. Defaults to '/login/token.php'.
            web_service_url (str, optional): Moodle Web Service endpoint. Defaults to '/webservice/rest/server.php'.

        Raises:
            InvalidCredentialException: Wrong credential

        Returns:
            Moodle: Returns an instance of Moodle
        """
        data = cls.get_tokens(domain, username, password, service, loginurl)
        if 'token' not in data:
            raise InvalidCredentialException()
        return cls(url=web_service_url if web_service_url.startswith(domain)
                   else domain + web_service_url,
                   token=data['token'])

    @classmethod
    def get_tokens(
        cls,
        domain: str,
        username: str,
        password: str,
        service: str = 'moodle_mobile_app',
        loginurl: str = '/login/token.php',
    ) -> dict:
        domain = domain.lstrip('/')
        params = {
            'username': username,
            'password': password,
            'service': service
        }
        url = loginurl if loginurl.startswith(domain) else domain + loginurl
        res = requests.get(url, params=params)
        return res.json() if res.ok else {}


class Moodle(Mdl):
    def __init__(self, url: str, token: str):
        super(Moodle, self).__init__(url, token)
        self._auth = Auth(self)
        self._core = Core(self)
        self._mod = Mod(self)
        self._tool = Tool(self)

    def __call__(self,
                 wsfunction: str,
                 moodlewsrestformat='json',
                 **kwargs) -> Any:
        return self.post(wsfunction, moodlewsrestformat, **kwargs)

    @property
    def auth(self) -> Auth:
        return self._auth

    @property
    def core(self) -> Core:
        return self._core

    @property
    def mod(self) -> Mod:
        return self._mod

    @property
    def tool(self) -> Tool:
        return self._tool



from typing import Callable, Type
from moodle import MoodleObject


def meta(wsfunction: str, moodlewsrestformat: str) -> Callable:
    def decorator(cls: Type[MoodleObject]) -> Type[MoodleObject]:
        cls.wsfunction = wsfunction
        cls.moodlewsrestformat = moodlewsrestformat
        return cls

    return decorator



from dataclasses import dataclass
from typing import List, Optional, Union
from moodle import Warning, ResponsesFactory


@dataclass
class AuthorUrls:
    """Author's urls
    Args:
        profile (Optional[str]): The URL for the use profile page
        profileimage (Optional[str]): The URL for the use profile image
    """
    profile: Optional[str]
    profileimage: Optional[str]


@dataclass
class GroupUrls:
    """Group's urls
    Args:
        image (Optional[str]): image
    """
    image: Optional[str]


@dataclass
class Group:
    """Group
    Args:
        id (int): id
        name (str): name
        urls (GroupUrls): urls
    """
    id: int
    name: str
    urls: GroupUrls


@dataclass
class PlagiarismHtml:
    """The HTML source for the Plagiarism Response
    Args:
        plagiarism (Optional[str]): The HTML source for the Plagiarism Response
    """
    plagiarism: Optional[str]


@dataclass
class AttachmentUrls:
    """Attachment's urls
    Args:
        export (Optional[str]): The URL used to export the attachment
    """
    export: Optional[str]


@dataclass
class TagUrls:
    """Tag's urls
    Args:
        view (str): The URL to view the tag
    """
    view: str


@dataclass
class RatingScaleItem:
    """Rating's Scale item
    Args:
        value (int): Scale value/option id.
        name (str): Scale name.
    """
    value: int
    name: str


@dataclass
class Rating:
    """Rating
    Args:
        itemid (int): Item id.
        scaleid (Optional[int]): Scale id.
        userid (Optional[int]): User who rated id.
        aggregate (Optional[float]): Aggregated ratings grade.
        aggregatestr (Optional[str]): Aggregated ratings as string.
        aggregatelabel (Optional[str]): The aggregation label.
        count (Optional[int]): Ratings count (used when aggregating).
        rating (Optional[int]): The rating the user gave.
        canrate (Optional[int]): Whether the user can rate the item.
        canviewaggregate (Optional[int]): Whether the user can view the aggregated grade.
    """
    itemid: int
    scaleid: Optional[int]
    userid: Optional[int]
    aggregate: Optional[float]
    aggregatestr: Optional[str]
    aggregatelabel: Optional[str]
    count: Optional[int]
    rating: Optional[int]
    canrate: Optional[int]
    canviewaggregate: Optional[int]


@dataclass
class RatingScale:
    """Rating's Scale
    Args:
        id (int): Scale id.
        courseid (Optional[int]): Course id.
        name (Optional[str]): Scale name (when a real scale is used).
        max (int): Max value for the scale.
        isnumeric (int): Whether is a numeric scale.
        items (List[RatingScaleItem]): Scale items. Only returned for not numerical scales.
    """
    id: int
    courseid: Optional[int]
    name: Optional[str]
    max: int
    isnumeric: int
    items: List[RatingScaleItem]


@dataclass
class RatingInfo:
    """Rating Info of Post
    Args:
        contextid (int): Context id.
        component (str): Context name.
        ratingarea (str): Rating area name.
        canviewall (Optional[int]): Whether the user can view all the individual ratings.
        canviewany (Optional[int]): Whether the user can view aggregate of ratings of others.
        scales (List[RatingScale]): Different scales used information
        ratings (List[Rating]): The ratings
    """
    contextid: int
    component: str
    ratingarea: str
    canviewall: Optional[int]
    canviewany: Optional[int]
    scales: List[RatingScale]
    ratings: List[Rating]


@dataclass
class Html:
    """Html source for post
    Args:
        rating (Optional[str]): The HTML source to rate the post
        taglist (Optional[str]): The HTML source to view the list of tags
        authorsubheading (Optional[str]): The HTML source to view the author details
    """
    rating: Optional[str]
    taglist: Optional[str]
    authorsubheading: Optional[str]


@dataclass
class Tag:
    """Tag of Post
    Args:
        id (int): The ID of the Tag
        tagid (int): The tagid
        isstandard (int): Whether this is a standard tag
        displayname (str): The display name of the tag
        flag (int): Wehther this tag is flagged
        urls (TagUrls): The URL to view the tag
    """
    id: int
    tagid: int
    isstandard: int
    displayname: str
    flag: int
    urls: TagUrls


@dataclass
class Attachment:
    """Attachment of Post
    Args:
        contextid (int): contextid
        component (str): component
        filearea (str): filearea
        itemid (int): itemid
        filepath (str): filepath
        filename (str): filename
        isdir (int): isdir
        isimage (int): isimage
        timemodified (int): timemodified
        timecreated (int): timecreated
        filesize (int): filesize
        author (str): author
        license (str): license
        filenameshort (str): filenameshort
        filesizeformatted (str): filesizeformatted
        icon (str): icon
        timecreatedformatted (str): timecreatedformatted
        timemodifiedformatted (str): timemodifiedformatted
        url (str): url
        urls (AttachmentUrls): export url
        html (PlagiarismHtml): The HTML source for the Plagiarism Response
    """
    contextid: int
    component: str
    filearea: str
    itemid: int
    filepath: str
    filename: str
    isdir: int
    isimage: int
    timemodified: int
    timecreated: int
    filesize: int
    author: str
    license: str
    filenameshort: str
    filesizeformatted: str
    icon: str
    timecreatedformatted: str
    timemodifiedformatted: str
    url: str
    urls: AttachmentUrls
    html: PlagiarismHtml


@dataclass
class PostUrls:
    """Post's urls
    Args:
        view (Optional[str]): The URL used to view the post
        viewisolated (Optional[str]): The URL used to view the post in isolation
        viewparent (Optional[str]): The URL used to view the parent of the post
        edit (Optional[str]): The URL used to edit the post
        delete (Optional[str]): The URL used to delete the post
        split (Optional[str]): The URL used to split the discussion with the selected post being the first post in the new discussion
        reply (Optional[str]): The URL used to reply to the post
        export (Optional[str]): The URL used to export the post
        markasread (Optional[str]): The URL used to mark the post as read
        markasunread (Optional[str]): The URL used to mark the post as unread
        discuss (Optional[str]): discuss
    """
    view: Optional[str]
    viewisolated: Optional[str]
    viewparent: Optional[str]
    edit: Optional[str]
    delete: Optional[str]
    split: Optional[str]
    reply: Optional[str]
    export: Optional[str]
    markasread: Optional[str]
    markasunread: Optional[str]
    discuss: Optional[str]


@dataclass
class Capability:
    """Capability of Post
    Args:
        view (int): Whether the user can view the post
        edit (int): Whether the user can edit the post
        delete (int): Whether the user can delete the post
        split (int): Whether the user can split the post
        reply (int): Whether the user can reply to the post
        selfenrol (int): Whether the user can self enrol into the course
        export (int): Whether the user can export the post
        controlreadstatus (int): Whether the user can control the read status of the post
        canreplyprivately (int): Whether the user can post a private reply
    """
    view: int
    edit: int
    delete: int
    split: int
    reply: int
    selfenrol: int
    export: int
    controlreadstatus: int
    canreplyprivately: int


@dataclass
class Author:
    """Author of Post
    Args:
        id (Optional[int]): id
        fullname (Optional[str]): fullname
        groups (List[Group]): groups
        urls (AuthorUrls): images
    """
    id: Optional[int]
    fullname: Optional[str]
    groups: List[Group]
    urls: AuthorUrls


@dataclass
class Post:
    """Post
    Args:
        id (int): id
        subject (str): subject
        replysubject (str): replysubject
        message (str): message
        messageformat (int): message format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        author (Author): Author
        discussionid (int): discussionid
        hasparent (int): hasparent
        parentid (Optional[int]): parentid
        timecreated (int): timecreated
        unread (Optional[int]): unread
        isdeleted (int): isdeleted
        isprivatereply (int): isprivatereply
        haswordcount (int): haswordcount
        wordcount (Optional[int]): wordcount
        capabilities (Capability): Capability
        urls (PostUrls): PostUrls
        attachments (List[Attachment]): attachments
        tags (List[Tag]): tags
        html (Html): html source
        ratinginfo (Optional[RatingInfo]): Rating information
    """
    id: int
    subject: str
    replysubject: str
    message: str
    messageformat: int
    author: Author
    discussionid: int
    hasparent: int
    parentid: Optional[int]
    timecreated: int
    unread: Optional[int]
    isdeleted: int
    isprivatereply: int
    haswordcount: int
    wordcount: Optional[int]
    capabilities: Capability
    urls: PostUrls
    attachments: List[Attachment]
    tags: List[Tag]
    html: Html
    ratinginfo: Optional[RatingInfo]


@dataclass
class Message:
    """Warning Message
    Args:
        type (str): The classification to be used in the client side
        message (str): untranslated english message to explain the warning
    """
    type: str
    message: str


@dataclass
class NewPost:
    """Response
    Args:
        postid (int): new post id
        warnings (List[Warning]): list of warnings
        post (Post): post object
        messages (List[Message]): list of warning messages
    """
    postid: int
    warnings: List[Warning]
    post: Post
    messages: List[Message]


@dataclass
class Posts(ResponsesFactory[Post]):
    """List of Post
    Args:
        posts (List[Post]): list of Post
        warnings (List[Warning]): list of Warning
    """
    posts: List[Post]
    warnings: List[Warning]

    @property
    def items(self) -> List[Post]:
        return self.posts

    @dataclass
    class Option:
        """Option for add_discussion_post
        Args:
            name (str): The allowed keys (value format) are:
                            discussionsubscribe (bool); subscribe to the discussion?, default to true
                            private             (bool); make this reply private to the author of the parent post, default to false.
                            inlineattachmentsid (int); the draft file area id for inline attachments
                            attachmentsid       (int); the draft file area id for attachments
                            topreferredformat   (bool); convert the message & messageformat to FORMAT_HTML, defaults to false
            value (str): the value of the option, this param is validated in the external function.
        """
        name: str
        value: Union[bool, int]



from dataclasses import dataclass
from typing import List
from moodle import Warning, ResponsesFactory


@dataclass
class Setting:
    """Mobile Setting
    Constructor arguments:
    params: name (str): The name of the setting
    params: value (str): The value of the setting
    """
    name: str
    value: str


@dataclass
class MobileConfig(ResponsesFactory[Setting]):
    settings: List[Setting]
    warning: List[Warning]

    @property
    def items(self) -> List[Setting]:
        return self.settings



from moodle import BaseMoodle, MessagePreference, NotificationPreference
from moodle.utils.helper import from_dict


class BaseMessage(BaseMoodle):
    def core_message_get_user_message_preferences(self) -> MessagePreference:
        data = self.moodle.get('core_message_get_user_message_preferences')
        return from_dict(MessagePreference, data)

    def get_user_notification_preferences(self) -> NotificationPreference:
        data = self.moodle.get(
            'core_message_get_user_notification_preferences')
        return from_dict(NotificationPreference, data)



from moodle import BaseMoodle


class BaseCohort(BaseMoodle):
    def add_cohort_members(self):
        data = self.moodle.get('core_cohort_add_cohort_members')
        return data

    def create_cohorts(self):
        data = self.moodle.get('core_cohort_create_cohorts')
        return data

    def delete_cohort_members(self):
        data = self.moodle.get('core_cohort_delete_cohort_members')
        return data

    def delete_cohorts(self):
        data = self.moodle.get('core_cohort_delete_cohorts')
        return data

    def get_cohort_members(self):
        data = self.moodle.get('core_cohort_get_cohort_members')
        return data

    def get_cohorts(self):
        data = self.moodle.get('core_cohort_get_cohorts')
        return data

    def search_cohorts(self):
        data = self.moodle.get('core_cohort_search_cohorts')
        return data

    def update_cohorts(self):
        data = self.moodle.get('core_cohort_update_cohorts')
        return data



from dataclasses import dataclass
from typing import List
from moodle import Warning


@dataclass
class View:
    """View Resource
    Args:
        status (int): status: true if success
        warnings (List[Warning]): list of warnings
    """
    status: int
    warnings: List[Warning]

    def __bool__(self) -> bool:
        if type(self.status) == bool:
            return bool(self.status)
        return self.status == 1



from datetime import datetime
from dataclasses import dataclass
from typing import List, Optional
from moodle import Warning, ResponsesFactory


@dataclass
class File:
    """File
    Args:
        filename (Optional[str]): File name.
        filepath (Optional[str]): File path.
        filesize (Optional[int]): File size.
        fileurl (Optional[str]): Downloadable file url.
        timemodified (Optional[int]): Time modified.
        mimetype (Optional[str]): File mime type.
        isexternalfile (Optional[int]): Whether is an external file.
        repositorytype (Optional[str]): The repository type for external files.
    """
    filename: Optional[str]
    filepath: Optional[str]
    filesize: Optional[int]
    fileurl: Optional[str]
    timemodified: Optional[int]
    mimetype: Optional[str]
    isexternalfile: Optional[int]
    repositorytype: Optional[str]


@dataclass
class Resource:
    """Resource
    Args:
        id (int): Module id
        coursemodule (int): Course module id
        course (int): Course id
        name (str): Page name
        intro (str): Summary
        introformat (int):  Default untuk "1" intro format (1 = HTML, 0 = MOODLE, 2 = PLAIN or 4 = MARKDOWN)
        introfiles (List[File]): Files in the introduction text
        contentfiles (List[File]): Files in the content
        tobemigrated (int): Whether this resource was migrated
        legacyfiles (int): Legacy files flag
        legacyfileslast (Optional[int]): Legacy files last control flag
        display (int): How to display the resource
        displayoptions (str): Display options (width, height)
        filterfiles (int): If filters should be applied to the resource content
        revision (int): Incremented when after each file changes, to avoid cache
        timemodified (datetime): Last time the resource was modified
        section (int): Course section id
        visible (int): Module visibility
        groupmode (int): Group mode
        groupingid (int): Grouping id
    """
    id: int
    coursemodule: int
    course: int
    name: str
    intro: str
    introformat: int
    introfiles: List[File]
    contentfiles: List[File]
    tobemigrated: int
    legacyfiles: int
    legacyfileslast: Optional[int]
    display: int
    displayoptions: str
    filterfiles: int
    revision: int
    timemodified: datetime
    section: int
    visible: int
    groupmode: int
    groupingid: int


@dataclass
class Resources(ResponsesFactory[Resource]):
    """List of Resource
    Args:
        resources (List[Resource]): List of Resource
        warnings (List[Warning]): List of Warning
    """
    resources: List[Resource]
    warnings: List[Warning]

    @property
    def items(self) -> List[Resource]:
        return self.resources



from dataclasses import dataclass
from typing import Optional


@dataclass
class EditorStructure:
    """Editor structure
    Args:
        text (str): The text for this feedback.
        format (int): The format for this feedback
    """
    text: str
    format: int


@dataclass
class PluginData:
    """Plugin Data
    Args:
        assignfeedbackcomments_editor (Optional[EditorStructure]): Editor structure
        files_filemanager (Optional[int]): The id of a draft area containing files for this feedback.
    """
    assignfeedbackcomments_editor: Optional[EditorStructure]
    files_filemanager: Optional[int]



from dataclasses import dataclass, field
from typing import List
from moodle import Warning


@dataclass
class CourseCompletionDetail:
    """Completion details
    Args:
        type (str): Type description
        criteria (str): Criteria description
        requirement (str): Requirement description
        status (str): Status description, can be anything
    """
    type: str
    criteria: str
    requirement: str
    status: str


@dataclass
class CourseCompletion:
    """Course Completion
    Args:
        type (int): Completion criteria type
        title (str): Completion criteria Title
        status (str): Completion status (Yes/No) a % or number
        complete (int): Completion status (true/false)
        timecompleted (int): Timestamp for criteria completetion
        details (CourseCompletionDetail): details
    """
    type: int
    title: str
    status: str
    complete: int
    timecompleted: int
    details: CourseCompletionDetail


@dataclass
class CompletionStatus:
    """Completion Status
    Args:
        completed (int): true if the course is complete, false otherwise
        aggregation (int): aggregation method 1 means all, 2 means any
        completions (List[CourseCompletion]): list of CourseCompletion
    """
    completed: int
    aggregation: int
    completions: List[CourseCompletion] = field(default_factory=list)


@dataclass
class CourseCompletionStatus:
    """Course Completion Status
    Args:
        completionstatus (CompletionStatus): Course status
        warnings (List[Warning]): list of warnings
    """
    completionstatus: CompletionStatus
    warnings: List[Warning]
